/********************************************************************************************************\

	File Name:	CommonDefs.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "CommonDefs.h"

/* converts shiplist damcon to in-game parts */
int DamCon_to_SpareParts( int DamCon )
{
	if (DamCon <= 2)
		return 2;
	if ((DamCon == 3) || (DamCon == 4))
		return 3;
	if ((DamCon == 5) || (DamCon == 6))
		return 4;
	if (DamCon == 7)
		return 5;
	if (DamCon == 8)
		return 6;
	if (DamCon == 9)
		return 7;
	if ((DamCon >= 10) && (DamCon <= 12))
		return 8;
	if ((DamCon >= 13) && (DamCon <= 15))
		return 9;
	if (DamCon >= 16)
		return 10;

	return 0;
}


// setfighters only works in OP
void RemoveFightersFromShip( tShipInfo* ship )
{
	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");

	debug_fprintf(fp, "Removing fighters from ship (%s)\n",
		ship->mGetClassName());
	debug_fprintf(fp, "   (preventing sudden fighter regen)\n");
#endif

#ifdef OP
	if (!(selected_enemy_sl.size())) 
		Populate_selected_enemy_sl();
	if (!(carry_over_fighter_sl.size()))
		Populate_carry_over_fighter_sl();

	// Load "aShip"
	Ship aShip;
	// Must have a shiplist that even non-hosts will prepare
	// in case they become hosts.
	selected_enemy_sl.Find(ship->mGetClassName(), aShip);

	// check for fighters.
	// up to 4 fighter bays
	int num_bays = 0;
	int32 count = 0;
	int32 total_num_fighters = 0;
	BaseShip fighter;
	Ship selected_fighter;
	ShipList tmpfighter_sl;

	for (int k = 0; k < 4; k++)
	{
		fighter = aShip.fighters[k].first;
		count = aShip.fighters[k].second;
		if (count)
		{
			num_bays++;
			total_num_fighters += count;
		}
	}

	// Must have a fighter shiplist even non-hosts will have
	// in case they do become the host.
	tmpfighter_sl = carry_over_fighter_sl;
	std::list< eRace >	tmpraces;
	tmpraces.push_back(aShip.race);
	tmpfighter_sl.Subset(tmpraces);
	if (tmpfighter_sl.size())
		tmpfighter_sl.SelectRandomShip(selected_fighter);

	if (total_num_fighters)
	{
		debug_fprintf(fp, "  (%s) should now have 0 fighters.\n",
			ship->mGetClassName());
		ship->mSetFighters(num_bays, 0, selected_fighter.shipCode);
	}
	
#else
	debug_fprintf(fp, "Sorry, SFC:EAW cannot reset fighter loadouts.\n");
#endif

#ifdef DEBUGMODE
	fclose(fp);
#endif
}


void Populate_selected_enemy_sl ( void )
{

	// load the shiplist.
	if (gScriptInterface.mIsHost() && (global_sl.size() == 0)) {
#ifdef OP
		global_sl.CreateMaster(ShipList::kOrionPirates, false);
#else
		global_sl.CreateMaster(ShipList::kEmpiresAtWar, false);
#endif
	}

	for (int l = (int)kTeam7; l <= (int)kTeam10; l++)
	{
		eTeamID team = (eTeamID)l;
		
		tTeamInfo *ti = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
		if (ti)
		{
			int32 i = ti->mGetShipsInPlayCount();		// can be -1... odd!

			//
			// If ship captured, must check current team
			//

			for (int k = 0; k < i; k++)
			{
				tShipInfo *	tsi = (tShipInfo *)ti->mGetThisShip(k+1);
				Ship aShip;
				global_sl.Find(tsi->mGetClassName(), aShip);

				selected_enemy_sl.push_back(aShip);
			}
		}
	}
}

void Populate_carry_over_fighter_sl ( void )
{
#ifdef OP
  carry_over_fighter_sl.CreateFighterList(ShipList::kOrionPirates, false);
#else
  carry_over_fighter_sl.CreateFighterList(ShipList::kEmpiresAtWar, false);
#endif
}


void debug_fprintf ( FILE* fp, const char* format, ... ) 
{
#ifdef DEBUGMODE
	va_list argptr ;
	va_start ( argptr, format ) ;
	int32 CurrentTimeCount = GetTickCount();

	// Insert timer counter here.
	char buf[512];
	sprintf(buf, "%d: %s\0", CurrentTimeCount - ZeroTimeCount, format);

	// actual printf here.
	vfprintf(fp, buf, argptr);
	va_end ( argptr ) ; 
	fflush(fp);
#endif
}

