#include "stdafx.h"
#include "CoopData.h"
#include <Windows.h>

CoopData gCoopData;

CoopData::CoopData()
{
	memset(this, 0, sizeof(CoopData));
}

CoopData::~CoopData()
{
}

#ifdef OP 
iTruth CoopData::FindSFCOPDirectory(std::string &location)
{
	HKEY	hKey;
	LONG	lval;
	DWORD	type;
	char	path[256];
	DWORD	siz = 256;

	lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						"Software\\Taldren\\Starfleet Command Orion Pirates",
						0,
						KEY_QUERY_VALUE,
						&hKey);

	if (lval != ERROR_SUCCESS)
		return kFalse;

	lval = RegQueryValueEx(	hKey,
							"Directory",
							0,
							&type,
							(unsigned char *)path,
							&siz );

	RegCloseKey(hKey);

	if (lval != ERROR_SUCCESS)
		return kFalse;

	path[siz] = 0;
	if (path[siz - 1] != '\\')
		strcat(path, "\\");

	location = path;
	return kTrue;
}

#else

iTruth CoopData::FindSFCEAWDirectory(std::string &location)
{
	HKEY	hKey;
	LONG	lval;
	DWORD	type;
	char	path[256];
	DWORD	siz = 256;

	lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						"Software\\Taldren\\Starfleet Command II",
						0,
						KEY_QUERY_VALUE,
						&hKey);

	if (lval != ERROR_SUCCESS)
		return kFalse;

	lval = RegQueryValueEx(	hKey,
							"Path",
							0,
							&type,
							(unsigned char *)path,
							&siz );

	RegCloseKey(hKey);

	if (lval != ERROR_SUCCESS)
		return kFalse;

	path[siz] = 0;
	if (path[siz - 1] != '\\')
		strcat(path, "\\");

	location = path;
	return kTrue;
}
#endif

iTruth CoopData::FindScriptsDirectory(std::string &location)
{
#ifdef OP
	iTruth t = FindSFCOPDirectory(location);
#else 
	iTruth t = FindSFCEAWDirectory(location);
#endif

	if (t == kTrue)
		location.append("Assets\\Scripts\\");

	return t;
}

iTruth	CoopData::Load(void)
{
	char buf[64];
	std::string	scriptsINI;
	if (!FindScriptsDirectory(scriptsINI))
		return kFalse;
	scriptsINI += "scripts.ini";

	DWORD	rv = GetPrivateProfileString(
					"Mul_CoopAce", "multiplier",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fEnemyBPVRatio = atof(buf);
	else
		fEnemyBPVRatio = 1.1f;			// DEFAULT: 1.10

	/* Game Type; Random or Selected races? */

	rv = GetPrivateProfileString("Mul_CoopAce", "type",
					"", buf, 64,
					scriptsINI.c_str());
	if (rv)
		fGametype = atoi(buf);
	else
		fGametype = 0; 						// DEFAULT: Random
					
	/* Array of random races to choose from */

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "rrace",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fRaces[Single] = atoi(buf);
	else
		fRaces[Single] = 0xFFFFFFFF;    // DEFAULT: ALL RACES

	/* Array of mix races to choose from */

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "srace",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fRaces[Mixed] = atoi(buf);
	else
		fRaces[Mixed] = 0xFFFFFFFF;    // DEFAULT: ALL RACES

	/* Array of hull classes to choose from */

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "hull",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fShipClasses = atoi(buf);
	else
		fShipClasses = 0xFFFFFFFF;		// DEFAULT: Enable All Hulls


	rv = GetPrivateProfileString("Mul_CoopAce", "era",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fEra = atoi(buf);
	else
#ifdef EAW
		fEra = kLate; 						// DEFAULT: Late
#endif
#ifdef OP
		fEra = kEra_Game_Use;				// OP DEFAULT: use game's settings
#endif


	/* Array of hull percentages */

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "hullpercentA",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		sHullPercent.dHullPercent[0] = atoi(buf);
	else
	{
		sHullPercent.fHullPercent[pFF] = ff_percentage;
		sHullPercent.fHullPercent[pDD] = dd_percentage;
		sHullPercent.fHullPercent[pDW] = wd_percentage;
		sHullPercent.fHullPercent[pCL] = cl_percentage;
	}
	rv = GetPrivateProfileString(
					"Mul_CoopAce", "hullpercentB",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		sHullPercent.dHullPercent[1] = atoi(buf);
	else
	{	sHullPercent.fHullPercent[pCA] = ca_percentage;
		sHullPercent.fHullPercent[pNCA] = nca_percentage;
		sHullPercent.fHullPercent[pBCH] = bch_percentage;
		sHullPercent.fHullPercent[pCV] = cv_percentage;
	}
	rv = GetPrivateProfileString(
					"Mul_CoopAce", "hullpercentC",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		sHullPercent.dHullPercent[2] = atoi(buf);
	else
	{	
		sHullPercent.fHullPercent[pDN] = dn_percentage;
		sHullPercent.fHullPercent[pBB] = bb_percentage;
		sHullPercent.fHullPercent[pMON] = monster_percentage;		// DEFAULT: Populate default percentages
		sHullPercent.fHullPercent[pNULL] = NULL; 
	}


	std::string	sfcINI;
#ifdef OP
	if (!FindSFCOPDirectory(sfcINI))
		return kFalse;
#else 
	if (!FindSFCEAWDirectory(sfcINI))
		return kFalse;
#endif
	sfcINI += "sfc.ini";

	rv = GetPrivateProfileString(
					"UI", "UIType",
					"", buf, 64,
					sfcINI.c_str());

	if (rv)
		fAIIntel = atof(buf);
	else
		fAIIntel = 0;					// NO DEFAULT...? Probably CAPTAIN = 0


#ifdef OP

	rv = GetPrivateProfileString(
					"Mul_CoopAce", "ModSupport",
					"", buf, 64,
					scriptsINI.c_str());

	if (rv)
		fModSupport = atof(buf);
	else
		fModSupport = 0xFFFFFFFF;		// Turn all flags by default;
#endif

	return kTrue;
}

iTruth CoopData::Save(void)
{
	char buf[64];

	std::string	scriptsINI;
	if (!FindScriptsDirectory(scriptsINI))
		return kFalse;
	scriptsINI += "scripts.ini";

	std::string	sfcINI;
#ifdef OP
	if (!FindSFCOPDirectory(sfcINI))
		return kFalse;
#else
	if (!FindSFCEAWDirectory(sfcINI))
		return kFalse;
#endif
	sfcINI += "sfc.ini";

	sprintf(buf, "%1.2f", fEnemyBPVRatio);
	if (!WritePrivateProfileString("Mul_CoopAce", "multiplier",
								   buf, scriptsINI.c_str()))
		return kFalse;

	sprintf(buf, "%d", fGametype);
	if (!WritePrivateProfileString("Mul_CoopAce", "type",
								   buf, scriptsINI.c_str()))
		return kFalse;

	sprintf(buf, "%d", fRaces[Single]);
	if (!WritePrivateProfileString("Mul_CoopAce", "rrace",
								   buf, scriptsINI.c_str()))
		return kFalse;
	
	sprintf(buf, "%d", fRaces[Mixed]);
	if (!WritePrivateProfileString("Mul_CoopAce", "srace",
								   buf, scriptsINI.c_str()))
		return kFalse;

	if (fGametype) {					// backwards compatibility
		if (!WritePrivateProfileString("Mul_CoopAce", "race",
								   buf, scriptsINI.c_str()))
			return kFalse;
	} else{
		sprintf(buf, "%d", fGametype);  // value 0 as in Random in the old customiser
		if (!WritePrivateProfileString("Mul_CoopAce", "race",
								   buf, scriptsINI.c_str()))
			return kFalse;
	}

	sprintf(buf, "%d", fShipClasses);
	if (!WritePrivateProfileString("Mul_CoopAce", "hull",
								   buf, scriptsINI.c_str()))
		return kFalse;
	sHullPercent.fHullPercent[pNULL]=NULL;
	sprintf(buf, "%d", sHullPercent.dHullPercent[0]);
	if (!WritePrivateProfileString("Mul_CoopAce", "hullpercentA",
								   buf, scriptsINI.c_str()))
		return kFalse;
	sprintf(buf, "%d", sHullPercent.dHullPercent[1]);
	if (!WritePrivateProfileString("Mul_CoopAce", "hullpercentB",
								   buf, scriptsINI.c_str()))
		return kFalse;
	sprintf(buf, "%d", sHullPercent.dHullPercent[2]);
	if (!WritePrivateProfileString("Mul_CoopAce", "hullpercentC",
								   buf, scriptsINI.c_str()))
		return kFalse;

sprintf(buf, "%d", fEra);
	if (!WritePrivateProfileString("Mul_CoopAce", "era",
								   buf, scriptsINI.c_str()))
		return kFalse;


	sprintf(buf, "%d", fAIIntel);
	if (!WritePrivateProfileString("UI", "UIType",
								   buf, sfcINI.c_str()))
		return kFalse;


#ifdef OP
	sprintf(buf, "%d", fModSupport);
	if (!WritePrivateProfileString("Mul_CoopAce", "ModSupport",
								   buf, scriptsINI.c_str()))
		return kFalse;
#endif


	return kTrue;
}

float	CoopData::mGetEnemyBPVRatio(void) { return fEnemyBPVRatio; }
void	CoopData::mSetEnemyBPVRatio(float f) { fEnemyBPVRatio = f; }

void	CoopData::mToggleGametype(void) { fGametype = !fGametype; };
void	CoopData::mSetGametype(int32u i) { fGametype = i; };

void	CoopData::mResetRaceSelections(void) { fRaces[fGametype] = 0;}

iTruth	CoopData::mIsRaceRandom(void) { return fGametype; }

iTruth	CoopData::mIsRaceSelected(eRace r) { return (fRaces[fGametype] & (1 << r)) != 0; }
void	CoopData::mSetRaceSelected(eRace r) { fRaces[fGametype] |= (1 << r); }

void	CoopData::mResetShipClassSelections(void) { fShipClasses = 0; }
iTruth	CoopData::mIsShipClassSelected(eClassTypes c) { return (fShipClasses & (1 << c)) != 0; }
void	CoopData::mSetShipClassSelected(eClassTypes c) { fShipClasses |= (1 << c); }
char	CoopData::mGetShipOdds(eOdds o) { return sHullPercent.fHullPercent[o];}

void	CoopData::mSetShipOdds(eOdds o, char v) {sHullPercent.fHullPercent[o] = v;}

int32u	CoopData::mGetRawRaceSelected (void) { return fRaces[fGametype]; }
int32u	CoopData::mGetRawShipClassSelected(void) { return fShipClasses; }

int32u	CoopData::mGetAIIntel(void) { return fAIIntel; }
void    CoopData::mSetAIIntel(int32u i) { fAIIntel = i; }

int32u	CoopData::mGetEra(void) { return fEra; }
void    CoopData::mSetEra(int32u i) { fEra = i; }



#ifdef OP
iTruth	CoopData::mGetX1Support(void) 
{ 
	if (fModSupport & kX1support) 
	{
		return TRUE;
	} else {
		return FALSE;
	}
}
void	CoopData::mSetX1Support(iTruth truth) 
{
	fModSupport |= kX1support;
	if (!truth) { 
		// set to zero .. ugly I know. But I don't know better.
		fModSupport -= kX1support;
	}
}

iTruth	CoopData::mGetXship_proportional(void)  
{  
	if (fModSupport & kXshipProportional) 
	{
		return TRUE;
	} else {
		return FALSE;
	}
}
void	CoopData::mSetXship_proportional(iTruth truth)  
{  
	fModSupport |= kXshipProportional;
	if (!truth) { 
		// set to zero .. ugly I know. But I don't know better.
		fModSupport -= kXshipProportional;  
	}
}

iTruth	CoopData::mGet_upgrade_AI_fighters(void)  
{  
	if (fModSupport & k_upgrade_AI_fighters) 
	{
		return TRUE;
	} else {
		return FALSE;
	}
}
void	CoopData::mSet_upgrade_AI_fighters(iTruth truth)  
{  
	fModSupport |= k_upgrade_AI_fighters;
	if (!truth) { 
		// set to zero .. ugly I know. But I don't know better.
		fModSupport -= k_upgrade_AI_fighters;  
	}
}

#endif