#ifndef coopdata_h
#define coopdata_h

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include "tclient.h"
#include "race.h"
#include "shipclasses.h"

// For EAW/OP detection
#include "soundnums.h"
#ifdef kSndPhaserFire
	#define EAW 1
#else
	#define OP 1
#endif

enum{
	Single=0,
	Mixed
};

enum
{
	kX1support=1 << 0,
	kXshipProportional=1 << 1,
	k_upgrade_AI_fighters=1 << 2,
};

enum
{
	kEarly = 0,
	kMiddle = 1,
	kLate = 2,
	kAdvanced = 3,
	kEra_Game_Use = 4,
};

enum eOdds{
	pFF=0,
	pDD,
	pDW,
	pCL,
	pCA,
	pNCA,
	pBCH,
	pCV,
	pDN,
	pBB,
	pMON,
	pNULL
};

#define monster_percentage 10
#define bb_percentage 10
#define dn_percentage 30
#define cv_percentage 20
#define bch_percentage 55
#define nca_percentage 45
#define ca_percentage 60
#define cl_percentage 65
#define wd_percentage 50
#define dd_percentage 70
#define ff_percentage 80

class CoopData
{
private:
	
	//
	// Variables
	//
	float	fEnemyBPVRatio;
	int32u	fGametype;  
	int32u	fShipClasses;	// 1 << eClassTypes as bitmap
	int32u  fAIIntel;		// Stored in sfc.ini [UI] UIType (Admiral=2 Captain=0)
	int32u	fRaces[2];			// 1 << eRace as bitmap
	union uHullPercent 
	{
		char	fHullPercent[12];
		int32u	dHullPercent[3];
	}sHullPercent;
	int32u	fEra;
	int32u	fModSupport;

	//
	// Used internally to find scripts.ini and sfc.ini
	//
	iTruth	FindScriptsDirectory(std::string &location);
#ifdef OP
	iTruth	FindSFCOPDirectory(std::string &location);
#else
	iTruth	FindSFCEAWDirectory(std::string &location);
#endif

public:

	CoopData();
	~CoopData();

	//
	// Read/Write information from the scripts.ini
	//

	iTruth	Load(void);
	iTruth	Save(void);

	//
	// Accessors
	//

	float	mGetEnemyBPVRatio(void);
	void	mSetEnemyBPVRatio(float);

	void	mToggleGametype(void);
	void	mSetGametype(int32u);

	void	mResetRaceSelections(void);
	/* same test */
	iTruth	mIsRaceRandom(void);
	void	mSetRaceRandom(eRace);
	iTruth	mIsRaceSelected(eRace);
	void	mSetRaceSelected(eRace);

	void	mResetShipClassSelections(void);
	iTruth	mIsShipClassSelected(eClassTypes);
	void	mSetShipClassSelected(eClassTypes);
	char	mGetShipOdds(eOdds);
	void	mSetShipOdds(eOdds, char);

	int32u	mGetRawRaceRandom(void);
	int32u	mGetRawRaceSelected(void);
	int32u	mGetRawShipClassSelected(void);

	int32u	mGetAIIntel(void);
	void	mSetAIIntel(int32u);


	int32u	mGetEra(void);
	void	mSetEra(int32u);

#ifdef OP
	iTruth	mGetX1Support(void);
	void	mSetX1Support(iTruth);
	iTruth	mGetXship_proportional(void);
	void	mSetXship_proportional(iTruth);
	iTruth	mGet_upgrade_AI_fighters(void);
	void	mSet_upgrade_AI_fighters(iTruth);
#endif
};

extern CoopData	gCoopData;

#endif