//	<Name>	tbasic.h

//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	universal data types, constants, macros


#pragma once

#ifndef	tbasic_h
#define	tbasic_h	1


//--------------------------------------------------------------------------------
//	Integral Types

#if	CompilerMetrowerks
	typedef	signed long long	int64;
	typedef	signed long long	int64s;
	typedef	unsigned long long	int64u;
#endif

#if	CompilerMicrosoft
	typedef	signed __int64		int64;
	typedef	signed __int64		int64s;
	typedef	unsigned __int64	int64u;
#endif

typedef	signed long		int32;
typedef	signed long		int32s;
typedef	unsigned long	int32u;

typedef	signed short	int16;
typedef	signed short	int16s;
typedef	unsigned short	int16u;

typedef	signed char		int8;
typedef	signed char		int8s;
typedef unsigned char	int8u;

	// signed quantities: counts, indices
typedef	int32s			iCount;
typedef	int32s			iIndex;

	// unsigned quantities: tags, booleans
typedef	int32u			iTag;
typedef	int8u			iTruth;

	// time (usually microseconds)
typedef	double			iTime;

// wide character data type definition, note that the test for the
// Borland compiler does not use the macro 'CompilerBorland' since that
// macro is defined in tLearn.h which is a header file that can not
// be guaranteed to have been included prior to -this- header file,
// hence the test for __BORLANDC__ instead, don't change that!
//
#ifdef __BORLANDC__
	typedef wchar_t		iWideChar;
#else
	typedef int16u		iWideChar;
#endif


	// constants
#define	kFalse	0
#define	kTrue	1

#ifndef	nil
	#define	nil ((void*)0L)
#endif


#ifndef NULL
#ifdef __cplusplus
#define NULL    0
#else
#define NULL    ((void *)0)
#endif
#endif




//--------------------------------------------------------------------------------
//	Endian Calculations and Conversions

//	Donovan's neat little trick to determine endian-ness at runtime.
static int16u    gTigerEndianWord = 0x0100;   // 1st byte is 0 if little, 1 if big-endian
inline iTruth gIsBigEndian(void)
{
    return( *((int8u *)(&gTigerEndianWord)) );
}

//	Integer packing and unpacking
#define gMAKEWORD(a, b)  (	(int16s)(((int8u)(a))	| (((int16s)((int8u)(b))) << 8))	)
#define gMAKELONG(a, b)  (	(int32s)(((int16s)(a))	| (((int32s)((int16s)(b))) << 16))	)
#define gLOWORD(l)       (	(int16s)(l)	)
#define gHIWORD(l)       (	(int16s)(((int32s)(l) >> 16) & 0xFFFF)	)
#define gLOBYTE(w)       (	(int8u)(w)	)
#define gHIBYTE(w)       (	(int8u)(((int16s)(w) >> 8) & 0xFF)	)
inline void gBswapWord(void *pInfo)
{
    int16s    tmp;

    tmp = *((int16s *)pInfo);
    *(int16s *)pInfo = gMAKEWORD( gHIBYTE(tmp), gLOBYTE(tmp) );
}
inline void gBswapDword(void *pInfo)
{
    int16s    tmpL,tmpH;
    int16s    *pWrd = (int16s *)pInfo;

    tmpL = pWrd[1];
    tmpH = pWrd[0];
    tmpL = gMAKEWORD( gHIBYTE(tmpL), gLOBYTE(tmpL) );
    tmpH = gMAKEWORD( gHIBYTE(tmpH), gLOBYTE(tmpH) );
    pWrd[0] = tmpL;
    pWrd[1] = tmpH;
}




//--------------------------------------------------------------------------------
//	Exceptions

class	GenericException {};
#ifdef assert						// someone may have already created a macro.
#undef assert
#endif
#define	assert(x)	{ if (!(x)) {throw GenericException();} }



//--------------------------------------------------------------------------------
// Complex Types


#ifndef _Q3BASE_H_
typedef struct tPoint tPoint;
struct tPoint
{
    short   x;
    short   y;
#ifdef  __cplusplus

	tPoint() {}

	tPoint( short _x, short _y ) : x( _x ), y( _y ) {}
	tPoint( const tPoint& point ) : x( point.x ), y( point.y ) {}

#endif
};



typedef struct tRect tRect;
struct tRect
{
    short   left;
    short   top;
    short   right;
    short   bottom;

#ifdef  __cplusplus

	inline tRect()
	{
		left = top = right = bottom = 0;
	}

	inline tRect(const tRect &T)
	{
		left	= T.left;
		top		= T.top;
		right	= T.right;
		bottom	= T.bottom;
	}

    inline tRect(int sx, int sy, int ex, int ey)
    {
        left    = (short)sx;
        top     = (short)sy;
        right   = (short)ex;
        bottom  = (short)ey;
    };

    inline void mSet(int sx, int sy, int ex, int ey)
    {
        left    = (short)sx;
        top     = (short)sy;
        right   = (short)ex;
        bottom  = (short)ey;
    };

	inline tRect & operator = (const tRect &T)
	{
		left	= T.left;
		top		= T.top;
		right	= T.right;
		bottom	= T.bottom;
		return *this;
	}

	inline iTruth operator == (const tRect &T) const
	{
		return	(
					(left	== T.left)	&&
					(top	== T.top)	&&
					(right	== T.right)	&&
					(bottom	== T.bottom)
				);
	}

	inline tRect & operator += (const tRect &T)
	{
		mUnion(&T);
		return *this;
	}

	inline tRect & operator + (const tRect &T)
	{
		tRect copy(*this);
		return (copy += T);
	}

	inline tRect & operator -= (const tRect &T)
	{
		mClip(&T);
		return *this;
	}

	inline tRect & operator - (const tRect &T)
	{
		tRect copy(*this);
		return (copy -= T);
	}

	inline iTruth operator < ( const tRect &T ) const
	{
		return (left < T.left);
	}

	inline iTruth operator > ( const tRect &T ) const
	{
		return T < *this;
	}

	inline iTruth operator <= ( const tRect &T ) const 
	{
		return !(T < *this);
	}

	inline iTruth operator >= ( const tRect &T ) const
	{
		return !(*this < T);
	}

    inline iTruth mIntersects(const tRect *pR)
    {
        return( (left < pR->right) && (top < pR->bottom)
            && (right > pR->left) && (bottom > pR->top) );

    };

    inline iTruth mIsEnclosedBy(const tRect *pE) const
    {
        return( (pE->left <= left) && (pE->top <= top)
            && (pE->right >= right) && (pE->bottom >= bottom) );
    };

    inline void mOffset(int deltaX, int deltaY)
    {
        left += (short)deltaX;  right  += (short)deltaX;
        top  += (short)deltaY;  bottom += (short)deltaY;
    };

    inline void mInset(int deltaX, int deltaY)
    {
        left += (short)deltaX;  right  -= (short)deltaX;
        top  += (short)deltaY;  bottom -= (short)deltaY;
    };

    inline iTruth mPointInside(int x, int y) const
    {
        return( (left <= (short)x) && (top <= (short)y)
            && (right > (short)x) && (bottom > (short)y) );
    };

    inline iTruth mEquals(const tRect *pR) const
    {
        return( (left == pR->right) && (top == pR->bottom)
            && (right == pR->left) && (bottom == pR->top) );
    };

    inline void mUnion(const tRect *pMerge)
    {
        if (pMerge->left < left)    left    = pMerge->left;
        if (pMerge->top < top)      top     = pMerge->top;
        if (pMerge->right > right)  right   = pMerge->right;
        if (pMerge->bottom > bottom)    bottom = pMerge->bottom;
    }

    inline void mClip(const tRect *pC)
    {
        if (left < pC->left)        left    = pC->left;
        if (top < pC->top)          top     = pC->top;
        if (right > pC->right)      right   = pC->right;
        if (bottom > pC->bottom)    bottom  = pC->bottom;
    };

    inline void mMakeClippedRect(const tRect *pClip, tRect *pNewRect) const
    {
        if (left < pClip->left)     pNewRect->left    = pClip->left;
        else                        pNewRect->left    = left;
        if (top < pClip->top)       pNewRect->top     = pClip->top;
        else                        pNewRect->top     = top;
        if (right > pClip->right)   pNewRect->right   = pClip->right;
        else                        pNewRect->right   = right;
        if (bottom > pClip->bottom) pNewRect->bottom  = pClip->bottom;
        else                        pNewRect->bottom  = bottom;
    };

    inline iTruth mIsEmpty(void) const
    {
        return( (left >= right) || (top >= bottom) );
    };

    inline int mWidth(void) const
    {
        int     wd = right - left;
        return( wd >= 0 ? wd : 0);
    };

    inline int mHeight(void) const
    {
        int     ht = bottom - top;
        return( ht >= 0 ? ht : 0);
    };

	inline tPoint mOrigin(void) const
	{
		tPoint	pt;
		pt.x = left;
		pt.y = top;
		return pt;
	};

	inline tPoint mExtent(void) const
	{
		tPoint	pt;
		pt.x = (short)mWidth();
		pt.y = (short)mHeight();
		return pt;
	};

#endif  // __cplusplus
};
#endif // Q3Base


#endif	// end tbasic_h block


//	<History>
//	28Sep95	rbarris		Started.
//	18Oct95	rbarris		typedefs, true and false
//	06Nov95	rbarris		"global"
//	10Jan97 rbarris		renamed and trimmed down
//	14Jan97 mhertogh	added nil define for MSVC++
//  18Mar97 mhertogh    added tRect
//	27Apr97	mgerard		added #undef for assert macro if already defined.
//	30Sep97	dmandap		Added 'iWChar' (wide character) data type def
//	10Oct97	mgerard		Added code for doing endian detection and conversion at runtime.
//	24Oct97	rliu		Changed type 'iWChar' to 'iWideChar'
//	01Jan98	rbarris		minor edits per guidelines
//	26Mar98	mgerard		added methods to the tRect struct for construction, assignment, comparison, etc...
//	10Apr98	mgerard		added lame-o comparator methods to tRect for VC5 STL compliance.
//	15Oct98	rbarris		added int64 typedefs for "long long".
