//	<Name>	tigerstl.h

//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	convenient access to STL

#pragma once

#ifndef	tigerstl_h
#define	tigerstl_h	1

#include "tlearn.h"


// shorthands
// T is a type
// CT is a container of a type
// OrdT is an ordering of a type, like less<int>

#if CompilerBorland

		// seq'd containers
	#define	xvector(T)					vector	<T>
	#define	xlist(T)					list	<T>
	#define	xdeque(T)					deque	<T>

		// assoc containers
	#define	xset(T,CT)					set< T, CT >
	#define	xmultiset(T,CT)				multiset< T, CT >

	#define	xmap(keyT,outT,OrdT)		map< keyT, outT, OrdT >
	#define	xmultimap(keyT,outT,OrdT)	multimap< keyT, outT, OrdT >

		// adaptors
	#define	xqueue(CT,T)				queue			< T, CT<T> >
	#define	xpriority_queue(CT,T,OrdT)	priority_queue	< T, CT<T>, OrdT >
	#define	xstack(CT,T)				stack			< T, CT<T> >


		// seq'd containers
	#include	<vector.h>
	#include	<deque.h>
	#include	<list.h>

		// assoc containers
	#include	<set.h>
	#include	<map.h>
	
		// adaptors
	#include	<stack.h>
	#include	<queue.h>

		// algorithm stuff
	// #include <algobase.h>
	// #include <algorithm.h>

#elif CompilerMetrowerks

	//
	//	Old MWerkz (Pro1)
	#if ( __MWERKS__ < 0x2200 )
	
			// seq'd containers
		#define	xvector(T)					vector	<T, allocator<T> >
		#define	xlist(T)					list	<T, allocator<T> >
		#define	xdeque(T)					deque	<T, allocator<T> >

			// assoc containers
		#define	xset(T,CT)					set< T, CT, allocator<T> >
		#define	xmultiset(T,CT)				multiset< T, CT, allocator<T> >

		#define	xmap(keyT,outT,OrdT)		map< keyT, outT, OrdT, allocator<outT> >
		#define	xmultimap(keyT,outT,OrdT)	multimap< keyT, outT, OrdT, allocator<outT> >

			// adaptors
		#define	xqueue(CT,T)				queue			< T, CT<T,allocator<T> >, allocator<T> >
		#define	xpriority_queue(CT,T,OrdT)	priority_queue	< T, CT<T,allocator<T> >, OrdT, allocator<T> >
		#define	xstack(CT,T)				stack			< T, CT<T,allocator<T> >, allocator<T> >

	//
	//	New MWerkz (Pro4)	
	#elif ( __MWERKS__ >= 0x2200 )

			// seq'd containers
		#define	xvector(T)					vector	<T>
		#define	xlist(T)					list	<T>
		#define	xdeque(T)					deque	<T>

			// assoc containers
		#define	xset(T,CT)					set< T, CT >
		#define	xmultiset(T,CT)				multiset< T, CT >

		#define	xmap(keyT,outT,OrdT)		map< keyT, outT, OrdT >
		#define	xmultimap(keyT,outT,OrdT)	multimap< keyT, outT, OrdT >

			// adaptors
		#define	xqueue(CT,T)				queue			< T, CT<T> >
		#define	xpriority_queue(CT,T,OrdT)	priority_queue	< T, CT<T>, OrdT >
		#define	xstack(CT,T)				stack			< T, CT<T> >

	#else
		#error Unable to recognize metrowerkz compiler version.
	#endif

		// seq'd containers
	#include	<vector.h>
	#include	<deque.h>
	#include	<list.h>

		// assoc containers
	#include	<set.h>
	#include	<map.h>
	
		// adaptors
	#include <stack.h>
	#include <queue.h>

		// algorithm stuff
	#include <algobase.h>
	#include <algorithm.h>

	// Somehow, one of the above header files ends up #define'ing bool!
	#ifdef bool
		#undef bool
	#endif

#elif	CompilerMicrosoft

		// seq'd containers
	#define	xvector(T)					std::vector	<T>
	#define	xlist(T)					std::list	<T>
	#define	xdeque(T)					std::deque	<T>

		// assoc containers
	#define	xset(T,CT)					std::set< T, CT >
	#define	xmultiset(T,CT)				std::multiset< T, CT >

	#define	xmap(keyT,outT,OrdT)		std::map< keyT, outT, OrdT >
	#define	xmultimap(keyT,outT,OrdT)	std::multimap< keyT, outT, OrdT >


// For MSVC4 _MSC_VER == 1000, for version 5 it's 1100
#if _MSC_VER>=1100
		// adaptors
	#define	xqueue(CT,T)				std::queue			< T, CT<T> >
	#define	xpriority_queue(CT,T,OrdT)	std::priority_queue	< T, CT<T>, OrdT >
	#define	xstack(CT,T)				std::stack			< T, CT<T> >

	#include <vector>
	#include <deque>
	#include <list>

		// assoc containers
	#include	<set>
	#include	<map>
	
		// adaptors
	#include <stack>
	#include <queue>

		// algorithm stuff
	#include <algorithm>

	// STL that comes with VC5 doesn't conflict with min/max unlike TG_STL
	#undef NOMINMAX
	#include <minmax.h>
#else
		// adaptors
	#define	xqueue(CT,T)				std::queue			< CT<T> >
	#define	xpriority_queue(CT,T,OrdT)	std::priority_queue	< T, CT<T>, OrdT >
	#define	xstack(CT,T)				std::stack			< CT<T> >

	// seq'd containers
	#include <vector.h>
	#include <deque.h>
	#include <list.h>

		// assoc containers
	#include	<set.h>
	#include	<map.h>
	#include	<multimap.h>
	
		// adaptors
	#include <stack.h>
//	#include <queue.h>		// Microsoft's Implementation does not have queue.h
							// All of the queue class info is in stack.h

		// algorithm stuff
	#include <algobase.h>
//	#include <algorithm.h>
	#include <algo.h>		// Microsoft calls it algo.h
#endif

#else
	#error	Unrecognized compiler.
#endif


	
//
//	Simplification macros (using deque<> and less<>)
//

#define		sxset(T)				xset( T , std::less< T > )
#define		sxmultiset(T)			xmultiset( T , std::less< T > )
#define		sxmap(keyT,outT)		xmap( keyT , outT , std::less< keyT > )
#define		sxmultimap(keyT,outT)	xmultimap( keyT , outT , std::less< keyT > )
#define		sxqueue(T)				xqueue( std::deque , T )
#define		sxpriority_queue(T)		xpriority_queue( std::deque , T , std::less< T > )
#define		sxstack(T)				xstack( std::deque , T )


#endif	// endif on tigerstl_h

//
//	<History>
//
// 25Feb97	rbarris		Created.
// 27Apr97	mgerard		Changed CompilerMicrosoft target to use new TG_STL headers.
// 14Jun97	rbarris		Removed conditional includes from CompilerMetrowerks clause. (algo vs algobase)
// 20Jun97	rbarris		Created new wrappers for STL stuff.
// 23Jun97	mgerard		Removed queue.h and renamed algorithm.h for CompilerMicrosoft.
// 27Jun97	mgerard		#undef bool for CompilerMetrowerks ("let's define an intrinsic type to be something else!")
// 27Jun97	mgerard		Changed calling convention for stack and queue for Microsoft (on RMeyer account)
// 06Aug97	mgerard		Added simplification meta-macros
// 08Aug97	bsnyder		VC5 items: Fixed adaptors and added queue include
// 03Sep98	mgerard		MWPro4 support.