//	<Name>	tlearn.h

//	<Legal>	Copyright (c) 1997 by Quicksilver Software, Inc. ALL RIGHTS RESERVED.
//	This document contains CONFIDENTIAL, PROPRIETARY, TRADE SECRET information
//	belonging to Quicksilver Software, Inc. and may not be reproduced, in whole
//	or in part, without prior written permission from Quicksilver Software, Inc.

//	<Info>	Assesses compiler, OS, and CPU type at build time


#ifndef	tlearn_h
#define	tlearn_h	1

#ifndef	__cplusplus
	#error C++ compiler must be activated.
#endif


	//	define all the symbols to 0 first
	// these are intended to be used with #if (not #ifdef!)
	// thus any of them can be  used by value in any circumstance.

	//	compiler symbols
#define	CompilerBorland			0
#define	CompilerMetrowerks		0
#define	CompilerOldMetrowerks	0
#define	CompilerMicrosoft		0
#define	CompilerWatcom			0

	//	chip symbols
#define	Chip68K				0
#define	ChipPPC				0
#define ChipX86				0

	//	environment symbols
#define	SystemMac7			0
#define	SystemWin32			0

#define	EndianBig			0
#define	EndianLittle		0



	//	Compiler and chip sensing


#ifdef		__BORLANDC__				// Borland sensing

	#undef	CompilerBorland
	#define	CompilerBorland			1

	#if	(_M_IX86 >= 300)			// same symbol as Microsoft VC,  under BC5

		#undef	ChipX86
		#define	ChipX86				1
		
		#ifdef	__WIN32__			// this differs from MSVC _WIN32.
			#undef	SystemWin32
			#define	SystemWin32		1
		#else
			#error	Borland: not Win32.
		#endif

	#else
		#error	Borland: not >=386.
	#endif

#endif


#ifdef	__MWERKS__					// Metrowerks sensing

	#pragma once

	#undef	CompilerMetrowerks
	#define	CompilerMetrowerks		1

	#if __MWERKS__ < 0x1100			// "Old metrowerks" has different STL requirements.  sigh.
		#undef	CompilerOldMetrowerks
		#define	CompilerOldMetrowerks 1
	#endif
		

	#ifdef	__MC68K__
	
		#undef	Chip68K
		#define	Chip68K				1
		
	#endif
		
	#ifdef	__POWERPC__
	
		#undef	ChipPPC
		#define	ChipPPC				1
		
	#endif
		
	#ifdef	__INTEL__
	
		#undef	ChipX86
		#define ChipX86				1
				
		#undef	SystemWin32
		#define	SystemWin32			1	//force Win32 until a better way is found

		#include <ansi_prefix.Win32.h>		
	#endif

#endif


#ifdef	_MSC_VER					// Microsoft sensing
	#ifndef	__MWERKS__					// Metrowerks also #defines _MSC_VER, so let's check
		#undef	CompilerMicrosoft
		#define	CompilerMicrosoft		1

													// Turn off unknown pragma warnings for
													// #pragma once when in MSVC++
		#pragma warning( disable: 4068 4786 4114 4236)
													// Also turn off warnings for "Identifier was truncated to '255' characters in the browser information"
													// and "same type qualifier used more than once"
													// and "'__export' is an obsolete keywork"
		#if	(_M_IX86 >= 300)

			#undef	ChipX86
			#define	ChipX86				1
			
			#ifdef	_WIN32
				#undef	SystemWin32
				#define	SystemWin32		1
			#else
				#error	Microsoft: not Win32.
			#endif

		#else

			#error	Microsoft: not >= 386.

		#endif
	#endif
#endif

#ifdef	__WATCOM__					//untested

	#undef	CompilerWatcom
	#define	CompilerWatcom			1

	#ifdef	__386__
	
		#undef	ChipX86
		#define	ChipX86				1

		#undef	SystemWin32
		#define	SystemWin32			1	//force Win32 until a better way is found
		
	#else
		#error	Watcom: chip not recognized.
	#endif
	
#endif


	// Ensure compiler was detected.

#if		CompilerBorland
#elif	CompilerMetrowerks
#elif	CompilerMicrosoft
#elif	CompilerWatcom
#else
	#error	Compiler not recognized.
#endif


	// Ensure chip was detected, set endian flags at the same time.

#if		Chip68K

	#undef	EndianBig
	#define	EndianBig		1
	
#elif	ChipPPC

	#undef	EndianBig
	#define	EndianBig		1
	
#elif	ChipX86

	#undef	EndianLittle
	#define	EndianLittle	1
	
#else
	#error	Chip not recognized.
#endif


	// Environment [OS] sensing

#if		Chip68K

	// assume Mac System 7
	
	#undef	SystemMac7
	#define	SystemMac7			1

#elif	ChipPPC

	// assume Mac System 7
	
	#undef	SystemMac7
	#define	SystemMac7			1
	
#elif	ChipX86

	// do nothing here
	
#else

	#error	Chip not recognized.
	
#endif


	// Ensure OS was detected, set endian flags at the same time.

#if		SystemMac7
#elif	SystemWin32
#else
	#error	Target OS not recognized.
#endif

#ifdef	UNICODE
	#error	UNICODE is not supposed to be defined.
#endif

#endif	// end tlearn_h block for file


//	<History>
//	22Sep95	rbarris		Started. Compiler, CPU, environment awareness mechanisms
//	27Sep95 rbarris		added sanity checks. these need to be kept up as we add targets/compilers.
//	31Dec96	rbarris		Heavily revised. Visual C++ added as a target. Testing needed.
//	03Jan97	rbarris		More Visual C++ work.
//	10Jan97 rbarris		Renamed "tlearn.h" for new naming.
//	04Mar97 gmarster	Added detection and disableing of #pragma once warnings in MSVC++
//	20Mar97	mgerard		Disabled 2 other useless MSVC warnings (see above for details)
//	27Apr97 mgerard		Disabled yet another lame MSVC warning
//	29May97	rbarris		Fixed confusion between MSVC and other compilers that #define _MSC_VER (CW12)
//	29May97	rbarris		Moved pragma once inside MW clause. Moved MSVC warning silencer inside MS clause.
//	09Jun97	rbarris		Eliminated Unicode option for Win32. Added check to make sure it's off.
//	16Jun97	mgerard		Added "CompilerOldMetrowerks" for the older STL style

