/********************************************************************************************************\

	File Name:	EnemyTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "math.h"
#include "tclient.h"
#include "CommonDefs.h"
#include "scriptinterface.h"
#include "Mul_CoopAce.h"
#include "worldconditions.h"

#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"

#include "HumanTeam.h"
#include "HumanTeamBaseState.h"

tClassAllocator<tEnemyTeam> gEnemyTeamAllocator;

static bool	FindScriptsDirectory(std::string &location)
{
	HKEY	hKey;
	LONG	lval;
	DWORD	type;
	char	path[256];
	DWORD	siz = 256;

#ifdef OP
	lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						"Software\\Taldren\\Starfleet Command Orion Pirates",
						0,
						KEY_QUERY_VALUE,
						&hKey);

	if (lval != ERROR_SUCCESS)
		return false;

	lval = RegQueryValueEx(	hKey,
							"Directory",
							0,
							&type,
							(unsigned char *)path,
							&siz );
#else
	lval = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
						"Software\\Taldren\\Starfleet Command II",
						0,
						KEY_QUERY_VALUE,
						&hKey);

	if (lval != ERROR_SUCCESS)
		return false;

	lval = RegQueryValueEx(	hKey,
							"Path",
							0,
							&type,
							(unsigned char *)path,
							&siz );
#endif


	RegCloseKey(hKey);

	if (lval != ERROR_SUCCESS)
		return false;

	path[siz] = 0;
	if (path[siz - 1] != '\\')
		strcat(path, "\\");

	strcat(path, "Assets\\Scripts\\");

	location = path;
	return true;
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tEnemyTeam::mSetupState()
{
	mRegisterState( new tEnemyTeamBaseState() );
	mGotoState( typeid( tEnemyTeamBaseState ) );
}

void tEnemyTeam::mSetupCommState()
{}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( static_cast< eTeamID >( kPlayerTeam ) );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPC1 ),
					nPCBPV,
					kClassFrigate,
					kClassFrigate,
					0,
					kStartPosition_Z,
					kPlayerNPC1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPC2 ), "PL3" , kStartPosition_J, kNoMetaShipID, kPlayerNPC2 );

*/

void tEnemyTeam::mCreateShipsForTeam()
{
	gCoopData.Load();
	mCreateCoopEnemy();					// was called from a timer but I didn't like the one-time lag
}

int numShips = 1;

ShipList	mon_sl;
ShipList	bb_sl;
ShipList	dn_sl;
ShipList	cv_sl;
ShipList	bch_sl;
ShipList	ca_sl;
ShipList	nca_sl;
ShipList	cl_sl;
ShipList	wd_sl;
ShipList	dd_sl;
ShipList	ff_sl;

//int32 latestShip = -999;
int32 earliestShip = 999 ;
int32 latestShip_available = -999;
int32 latestShip_retired = 999;


#define		MAX_ASSIGN	8
#define		MAX_SHIPS	32
int32 cheapest_ship = 0;

void tEnemyTeam::mCreateCoopEnemy()		// called from timer
{
	FILE *		fp = NULL;
#ifdef DEBUGMODE
	if (mGetTeam() == kTeam7) 
	{
		fp = fopen("C:\\Coop.txt", "w");
		ZeroTimeCount = GetTickCount();

		debug_fprintf(fp, "Opening coop.txt logfile.\n");
	}
	else 
	{
		fp = fopen("C:\\Coop.txt", "a");
	}
#endif
	
	if (!gScriptInterface.mIsHost())
		return;							// not host, doesn't matter

	eTeamID current_team = mGetTeam();
	ShipList	sl;
	ShipList	fighter_sl;

	float x1_ship_prob = 0;
	float x2_ship_prob = 0;



	if (current_team == kTeam7) 
	{
		// Move back to "Mul_CoopAce.cpp" if all clients needs this.
		// Let the host load the shiplist

		if (global_sl.size() == 0) {
#ifdef OP
			global_sl.CreateMaster(ShipList::kOrionPirates, false);
#else
			global_sl.CreateMaster(ShipList::kEmpiresAtWar, false);
#endif
		}

		// The shiplist will be processed in 2 parts.
		// 1- sl = global_sl; remove excess races and hullclasses  (removals)
		// 2- carry_over_sl will be populated while processing sl's for era.  (additions)
		sl = global_sl; 

		// The following is much faster than rereading the whole files to have a ShipList
		// for fighters/PFs (and not just the less useful RealShipList)
		{
		RealShipList fsl = sl.FighterList();
		SHIP_LIST_ITER(fsl)
			fighter_sl.push_back(*iter);
		SHIP_LIST_ITER_END
		}


#ifdef OP
		debug_fprintf(fp, "Co-op Ace 3.21 for SFC:OP\n");
#else
		debug_fprintf(fp, "Co-op Ace 3.21 for SFC:EAW\n");
#endif

		debug_fprintf(fp, "First Enemy Team (kTeam7): initializing Co-op System...\n");
		debug_fprintf(fp, "There are %d ships in the shiplist\n", sl.size());

		std::string	scriptsINI;
		FindScriptsDirectory(scriptsINI);
		scriptsINI.append("scripts.ini");

		debug_fprintf(fp, "host set bpv multiplier = %1.2f\n", gCoopData.mGetEnemyBPVRatio());
		debug_fprintf(fp, "host set races = %d\n", gCoopData.mGetRawRaceSelected());
		debug_fprintf(fp, "host set hulls = %d\n", gCoopData.mGetRawShipClassSelected());


		int32	totalPlayerBPV = 0;
		int32	totalShipCount = 0;
		int32	thisTeamShipCount = 0;

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		{
			thisTeamShipCount = 0;

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
			if (ti)
			{
				tShipIterator tsi = ti->mGetFirstShip();
				for (;;)
				{
					tShipInfo *si = *tsi;
					if (!si)
						break;

				// mGetClassName returns invalid pointer here
				//	debug_fprintf(fp, "Found a ship belonging to team %d: '%s'\n" /* bpv %d.\n" */, ((int)team) + 1, si->mGetClassName()); //, si->mGetBPV());

					thisTeamShipCount++;
				//	totalPlayerBPV += si->mGetBPV(); doesn't work, always returns 0
				//	totalPlayerBPV += si->mGetTotalBPV(); doesn't work, always returns 0
					tsi++;
				}

				totalShipCount += thisTeamShipCount;

				if (thisTeamShipCount)
				{
					debug_fprintf(fp, "Team %d has %d ships totalling %d combat BPV (not including loadouts).\n", 
						((int)team) + 1, thisTeamShipCount,	ti->mGetCombatBPV());
				//	totalPlayerBPV += ti->mGetTotalCombatBPV(); doesn't work, always returns 0
					totalPlayerBPV += ti->mGetCombatBPV(); // doesn't include loadouts :<
				}
			}
		}

		debug_fprintf(fp, "Total Player BPV is %d.\n\n", totalPlayerBPV);

		totalPlayerBPV *= gCoopData.mGetEnemyBPVRatio();
		bpv_to_assign = totalPlayerBPV;

		debug_fprintf(fp, "Total Enemy BPV is %d.\n\n", totalPlayerBPV);

#ifdef OP
		int32 selected_era = fMissionInfo->mGetEra();
		if (gCoopData.mGetEra() <= kAdvanced) {
			selected_era = gCoopData.mGetEra();
			debug_fprintf(fp, "*** Era forced from customizer.\n");
		}
#else
		int32 selected_era = gCoopData.mGetEra();
#endif
		int32 selected_era_end_year = 999;
		int32 selected_era_begin_year = -999;

		if (selected_era == kEarly)
		{
			selected_era_end_year = kEEnd;
			selected_era_begin_year = kEBegin;
			debug_fprintf(fp, "Selected Era: Early.\n");
		}
		else if (selected_era == kMiddle)
		{
			selected_era_end_year = kMEnd;
			selected_era_begin_year = kMBegin;
			debug_fprintf(fp, "Selected Era: Middle.\n");
		}
		else if (selected_era == kLate)
		{
			selected_era_end_year = kLEnd;
			selected_era_begin_year = kLBegin;
			debug_fprintf(fp, "Selected Era: Late.\n");
		}
		else if (selected_era == kAdvanced)
		{
			selected_era_end_year = kAEnd;
			selected_era_begin_year = kABegin;
			debug_fprintf(fp, "Selected Era: Advanced.\n");
		}

		/*
		 * Calculate era and limit ships to that era and earlier
		 */

		Ship s;

		float num_human_x1 = 0;
		float num_human_x2 = 0;
		float num_human_ships = 0;
		for (PlayerShipClassNameList::iterator pi = pscnl.begin(); pi != pscnl.end(); pi++)
		{
			if (sl.Find(*pi, s))
			{
				debug_fprintf(fp, "PlayerShipClassName %s (yfa=%d yla=%d) ", 
					pi->c_str(), s.yearFirstAvail, s.yearLastAvail);	
				num_human_ships++;

				// check for X2 ships
				if (s.yearFirstAvail >= kABegin) {
					debug_fprintf(fp, "ADVANCED\n");
					num_human_x2++;
				} else
				// check for X1 ships
				if ((s.yearFirstAvail >= kX1Begin) && 
					(s.yearFirstAvail < kABegin)) 
				{
					debug_fprintf(fp, "LATE --- X1\n");
					num_human_x1++;
				} else if (s.yearFirstAvail >= kLBegin) {
					debug_fprintf(fp, "LATE\n");
				}
				else if (s.yearFirstAvail >= kMBegin) {
					debug_fprintf(fp, "MID\n");
				}
				else 
				{
					debug_fprintf(fp, "EARLY\n");
				}

				if (s.yearFirstAvail < earliestShip)
					earliestShip = s.yearFirstAvail;
				if (s.yearFirstAvail > latestShip) {
					latestShip = s.yearFirstAvail;
					latestShip_available = s.yearFirstAvail;
					latestShip_retired = s.yearLastAvail;
				}
			}
		}

#ifdef OP
		// Will be used to decide prob of xships, later
		if (gCoopData.mGetXship_proportional()) 
		{
			x2_ship_prob = num_human_x2 / num_human_ships;
			debug_fprintf(fp, "Increased X2 probability switch is ON.\n");
			debug_fprintf(fp, "%.1f x2ships out of %.1f total human-controlled ships.\n",
				num_human_x2, num_human_ships);
		} else {
			x2_ship_prob = 0;
			debug_fprintf(fp, "Increased X2 probability switch is OFF.\n");
		}

		if (x2_ship_prob > 0) {
			debug_fprintf(fp, "%.2f prob enemy will be ALL x2ships\n",
				x2_ship_prob);
		}

		
		if (gCoopData.mGetXship_proportional()) 
		{
			x1_ship_prob = num_human_x1 / num_human_ships;
			debug_fprintf(fp, "Increased X1 probability switch is ON.\n");
			debug_fprintf(fp, "%.1f x1ships out of %.1f total human-controlled ships.\n",
				num_human_x1, num_human_ships);
		} else {
			x1_ship_prob = 0;
			debug_fprintf(fp, "Increased X1 probability switch is OFF.\n");
		}
		if (! gCoopData.mGetX1Support()) {
			x1_ship_prob = 0;
			debug_fprintf(fp, "Support for separate X1 era is OFF.\n");
		}
		if (x1_ship_prob > 0) {
			debug_fprintf(fp, "%.2f prob enemy will be ALL x1ships\n",
				x1_ship_prob);
		}

		if (gCoopData.mGetXship_proportional() 
			&& ((x1_ship_prob + x2_ship_prob) > 0)) 
		{
			debug_fprintf(fp, "%.2f prob enemy will be ALL x1 and X2 ships, mixed.\n",
				x1_ship_prob + x2_ship_prob);
		}
#endif //OP



		debug_fprintf(fp, "latestShip year: %d\n", latestShip);


#ifndef ERA_YEAR_BASED
		if (latestShip >= kABegin)
		{
		    selected_era_end_year = kAEnd;
		}
		else if (latestShip >= kX1Begin)
		{			// X1 era
			selected_era_end_year = kX1End;
		}
		else if (latestShip >= kLBegin)
		{			// late era - do nothing
			selected_era_end_year = kLEnd;
		}
		else if (latestShip >= kMBegin)
		{			// mid era - strip off late
			selected_era_end_year = kMEnd;
		}
		else
		{			// early era - strip off mid, late
			selected_era_end_year = kEEnd;
		}
#endif


		// use a separate value in case we need a separate first availability date
		int32 latestShip_first_available = latestShip_available;

#ifndef ERA_YEAR_BASED
		// if we're selecting in an Era, make sure all from
		// beginning of that era is selected.
		// Please note that X1 is handled differently. 
		// As of OP2550, LEnd and X1end are the same.
		if (latestShip_first_available <= kEEnd)
			latestShip_first_available = kEBegin;
		else if (latestShip_first_available <= kMEnd)
			latestShip_first_available = kMBegin;
		// else if (latestShip_first_available < kLEnd)
		else if (latestShip_first_available < kX1Begin)
			latestShip_first_available = kLBegin;
		// added for X1-only games
		else if (latestShip_first_available <= kX1End)
			latestShip_first_available = kX1Begin;
		else if (latestShip_first_available <= kAEnd)
			latestShip_first_available = kABegin;
#else
		latestShip_first_available = selected_era_begin_year;
#endif



		std::list< eRace >	races;
		if (!gCoopData.mIsRaceRandom())  // 0 is random type, 1 is selected type
		{
			eRace randomrace;
			// Problems with kOrion neutrals. Previous is kOrionCamboro
			//eRace randomrace = (eRace)fMissionInfo->mRandomInt32(kFederation, kOrion);
			//eRace randomrace = (eRace)fMissionInfo->mRandomInt32(kFederation, kOrionCamboro);

			if (gCoopData.mGetRawRaceSelected() > 0)
			{
				int race_array[(int)kMonster];
				int race_counter = 0;
				for (int i = 0; i <= kMonster; i++) {
					if (gCoopData.mIsRaceSelected((eRace) i)) {
						race_array[race_counter] = i;
						race_counter++;
					}
				}

				//while(!gCoopData.mIsRaceSelected(randomrace)) {
				//	randomrace = (eRace)fMissionInfo->mRandomInt32(kFederation, kMonster);
				//} 
				randomrace = (eRace)race_array[fMissionInfo->mRandomInt32(0, (race_counter -1))];
				races.push_back(randomrace);
				// was through kMonster, if kMonster then monster ship class required
				// decided to remove kMonster possibility from "single race".
			} else {
				randomrace = kNoRace;
			}
			debug_fprintf(fp, "Race chosen: %d\n", (int)(*(races.begin())));
		}
		else
		{
			if (gCoopData.mIsRaceSelected(kFederation))	{ 
				races.push_back(kFederation); 
				debug_fprintf(fp, "Race Federation Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kKlingon))	{ 
				races.push_back(kKlingon);	
				debug_fprintf(fp, "Race Klingon Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kRomulan))	{ 
				races.push_back(kRomulan);	
				debug_fprintf(fp, "Race Romulan Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kGorn))		{ 
				races.push_back(kGorn);		
				debug_fprintf(fp, "Race Gorn Available.\n");
			}
			if (gCoopData.mIsRaceSelected(kLyran))		{ 
				races.push_back(kLyran);		
				debug_fprintf(fp, "Race Lyran Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kHydran))		{ 
				races.push_back(kHydran);
				debug_fprintf(fp, "Race Hydran Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kISC))		{ 
				races.push_back(kISC);		
				debug_fprintf(fp, "Race ISC Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kMirak))		{ 
				races.push_back(kMirak);		
				debug_fprintf(fp, "Race Mirak Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kMonster))	{ 
				races.push_back(kMonster);	
				debug_fprintf(fp, "Race Monster Available.\n"); 
			}
			if (gCoopData.mIsRaceSelected(kOrion))		{ 
				races.push_back(kOrion);
				debug_fprintf(fp, "Non-Aligned Pirates Available.\n");
			}
#ifdef OP
			if (gCoopData.mIsRaceSelected(kOrionOrion))		{
				races.push_back(kOrionOrion);
				debug_fprintf(fp, "Cartel Orion Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionKorgath))		{
				races.push_back(kOrionKorgath);
				debug_fprintf(fp, "House of Korgath Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionPrime))		{
				races.push_back(kOrionPrime);
				debug_fprintf(fp, "Prime Industries Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionTigerHeart))		{
				races.push_back(kOrionTigerHeart);
				debug_fprintf(fp, "Cartel Tiger Heart Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionBeastRaiders))		{
				races.push_back(kOrionBeastRaiders);
				debug_fprintf(fp, "Cartel Beast Raiders Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionSyndicate))		{
				races.push_back(kOrionSyndicate);
				debug_fprintf(fp, "Cartel Syndicate Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionWyldeFire))		{
				races.push_back(kOrionWyldeFire);
				debug_fprintf(fp, "WyldeFire Combine Available.\n");
			} 
			if (gCoopData.mIsRaceSelected(kOrionCamboro))		{
				races.push_back(kOrionCamboro);
				debug_fprintf(fp, "Cartel Camboro Available.\n"); 
			}
#endif
		}

		debug_fprintf(fp, "Running races subsets now. (currently %d ships)\n", sl.size());

		sl.Subset(races);
		debug_fprintf(fp, "Done races/ships subset. (currently %d ships)\n", sl.size());

		fighter_sl.Subset(races);
		debug_fprintf(fp, "Done races/fighter subset.\n");

		debug_fprintf(fp, "Preparing hullclass subsets\n");
		std::list< eClassTypes >	classes;

		if (gCoopData.mIsShipClassSelected(kClassFrigate))
		{ 
			classes.push_back(kClassFrigate);
			debug_fprintf(fp, "Hull Frigate Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassDestroyer))			
		{ 
			classes.push_back(kClassDestroyer);	
			debug_fprintf(fp, "Hull Destroyer Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassWarDestroyer))			
		{ 
			classes.push_back(kClassWarDestroyer);
			debug_fprintf(fp, "Hull War Destroyer Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassLightCruiser))			
		{ 
			classes.push_back(kClassLightCruiser);
			debug_fprintf(fp, "Hull Light Cruiser Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassHeavyCruiser))			
		{ 
			classes.push_back(kClassHeavyCruiser);
			debug_fprintf(fp, "Hull Heavy Cruiser Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassNewHeavyCruiser))		
		{ 
			classes.push_back(kClassNewHeavyCruiser);
			debug_fprintf(fp, "Hull New Heavy Cruiser Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassHeavyBattlecruiser))	
		{ 
			classes.push_back(kClassHeavyBattlecruiser);
			debug_fprintf(fp, "Hull Heavy Battlecruiser Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassCarrier))				
		{ 
			classes.push_back(kClassCarrier);
			debug_fprintf(fp, "Hull Carrier Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassDreadnought))			
		{ 
			classes.push_back(kClassDreadnought);
			debug_fprintf(fp, "Hull Dreadnought Available.\n"); 
		}
		if (gCoopData.mIsShipClassSelected(kClassBattleship))			
		{ 
			classes.push_back(kClassBattleship);
			debug_fprintf(fp, "Hull Battleship Available.\n"); 
		}
		if (gCoopData.mIsRaceSelected(kMonster))						
		{ 
			classes.push_back(kClassMonster);
			debug_fprintf(fp, "Hull Monster Available.\n"); 
		}

		debug_fprintf(fp, "Running hullclass subset. (currently %d ships)\n", sl.size());

		sl.Subset(classes);
		debug_fprintf(fp, "Done hullclass subset. (currently %d ships)\n", sl.size());

		debug_fprintf(fp, "Selecting ships in existance between SFC year %d and %d\n", 
			latestShip_first_available, selected_era_end_year);

		// using data only from 'Latest' because this *IS* a coop-ace script. 
		//   After all, "Are you good enough?"
		

		int32 fighter_date_end = kAEnd;

#ifdef OP
		if (latestShip_first_available >= kABegin)
		{	//advanced
			{
				int32 tmp_prob_x2 = (int32 )(floor(x2_ship_prob*100));
				int32 tmp_prob_x1 = (int32 )(floor(x1_ship_prob*100));
				int32 random100 = fMissionInfo->mRandomInt32(0, 100);

				debug_fprintf(fp, "Advanced era code branched. Random value: %d\n", 
					random100);

				if (random100 < tmp_prob_x2) {
					// remove early, mid and late ships. Final.
					debug_fprintf(fp, "** NOTE: ALL enemy ships will be x2-ships.\n");

					SHIP_LIST_ITER(sl)
					if (
						(!(iter->yearFirstAvail < kABegin)) &&
						(!(iter->yearFirstAvail == 999))
						)
					{
						carry_over_sl.push_back(*iter);
						if (!cheapest_ship) 
						{
							cheapest_ship = iter->bpv;
						} else {
						if (iter->bpv < cheapest_ship)
							cheapest_ship = iter->bpv;
						}
					}
					SHIP_LIST_ITER_END
				} else 	if (
					(tmp_prob_x2 > 0) 
					&& (random100 < (tmp_prob_x2 + tmp_prob_x1))
					&& gCoopData.mGetX1Support()
				) {

					// remove early, mid and late ships. Final.
					debug_fprintf(fp, 
						"** NOTE: ALL enemy ships will be x1 and x2-ships, mixed.\n");

					SHIP_LIST_ITER(sl)
					if (
						(!(iter->yearFirstAvail < kX1Begin)) &&
						(!(iter->yearFirstAvail == 999))
						)
					{
						carry_over_sl.push_back(*iter);
						if (!cheapest_ship) 
						{
							cheapest_ship = iter->bpv;
						} else {
						if (iter->bpv < cheapest_ship)
							cheapest_ship = iter->bpv;
						}
					}
					SHIP_LIST_ITER_END

				} else {

					// only remove ships not in service anymore.
					debug_fprintf(fp, 
						"** NOTE: mixed fleet: only remove ships not in service anymore.\n");

					SHIP_LIST_ITER(sl)
					if (
						(!(iter->yearLastAvail < kABegin)) &&
						(!(iter->yearFirstAvail == 999))
						)
					{
						carry_over_sl.push_back(*iter);
						if (!cheapest_ship) 
						{
							cheapest_ship = iter->bpv;
						} else {
						if (iter->bpv < cheapest_ship)
							cheapest_ship = iter->bpv;
						}
					}
					SHIP_LIST_ITER_END
				}
			}
		}
		else if (
			(latestShip_available >= kX1Begin)
			&& gCoopData.mGetX1Support()
			&& (gCoopData.mGetEra() >= kLate)
			)
		{	//X1
			{
				int32 tmp_prob_x1 = (int32 )(floor(x1_ship_prob*100));
				int32 random100 = fMissionInfo->mRandomInt32(0, 100);

				debug_fprintf(fp, "X1 era code branched. Random value: %d\n", 
					random100);

				if (
					gCoopData.mGetX1Support()
					&& (random100 < tmp_prob_x1) 
					)
				{

					// remove early, mid and late ships. Final.
					debug_fprintf(fp, "** NOTE: ALL enemy ships will be x1-ships.\n");

					SHIP_LIST_ITER(sl)
					if (
						(!(iter->yearFirstAvail < kX1Begin)) &&
						(!(iter->yearFirstAvail > kX1End))
						)
					{
						carry_over_sl.push_back(*iter);

						if (!cheapest_ship) 
						{
							cheapest_ship = iter->bpv;
						} else {
						if (iter->bpv < cheapest_ship)
							cheapest_ship = iter->bpv;
						}
					}
					SHIP_LIST_ITER_END
				} else {
					// only remove ships not in service anymore.
					debug_fprintf(fp, 
						"** NOTE: mixed fleet: only remove ships not in service anymore.\n");

					SHIP_LIST_ITER(sl)
					if (
						(!(iter->yearLastAvail < kX1Begin)) &&
						(!(iter->yearFirstAvail > kX1End))
						)
					{
						carry_over_sl.push_back(*iter);

						if (!cheapest_ship) 
						{
							cheapest_ship = iter->bpv;
						} else {
						if (iter->bpv < cheapest_ship)
							cheapest_ship = iter->bpv;
						}
					}
					SHIP_LIST_ITER_END
				}
			}
		}
		else 

#endif // OP
		if (latestShip_first_available >= kLBegin)
		{	// late
			// remove early and mid ships not in use anymore
			// remove advanced ships
			{
				debug_fprintf(fp, "Late era code branched.\n");

				SHIP_LIST_ITER(sl)
				if (!
					((iter->yearLastAvail < kLBegin) || 
					(iter->yearFirstAvail > selected_era_end_year))
					) {
					carry_over_sl.push_back(*iter);

					if (!cheapest_ship) 
					{
						cheapest_ship = iter->bpv;
					} else {
					if (iter->bpv < cheapest_ship)
						cheapest_ship = iter->bpv;
					}
				}
				SHIP_LIST_ITER_END
			}
			fighter_date_end = kLEnd;
		}
		else if (latestShip_first_available >= kMBegin)
		{	// mid
			// remove early ships not in use anymore
			// remove late and advanced ships
			{
				debug_fprintf(fp, "Mid era code branched.\n");

				SHIP_LIST_ITER(sl)
				if (!
					((iter->yearLastAvail < kMBegin) || 
					(iter->yearFirstAvail > selected_era_end_year))
				    ) {
					carry_over_sl.push_back(*iter);

					if (!cheapest_ship) 
					{
						cheapest_ship = iter->bpv;
					} else {
					if (iter->bpv < cheapest_ship)
						cheapest_ship = iter->bpv;
					}
				}
				SHIP_LIST_ITER_END
			}
			fighter_date_end = kMEnd;
		}
		else
		{	// early
			// remove mid, late and advanced
			{
				debug_fprintf(fp, "early era code branched.\n");

				SHIP_LIST_ITER(sl)
				if (!(iter->yearFirstAvail > selected_era_end_year)) {
					carry_over_sl.push_back(*iter);

					if (!cheapest_ship) 
					{
						cheapest_ship = iter->bpv;
					} else {
					if (iter->bpv < cheapest_ship)
						cheapest_ship = iter->bpv;
					}
				}
				SHIP_LIST_ITER_END
			}
			fighter_date_end = kEEnd;
		}


		debug_fprintf(fp, "Done era code branching.\n");


#ifdef DEBUGMODE
		debug_fprintf(fp, "\nShips Available:\n\n");
		debug_fprintf(fp, "Code      BPV  YFA YLA\n");
		{
		SHIP_LIST_ITER(carry_over_sl)
			debug_fprintf(fp, "%-8s %4d %4d %3d\n",
				iter->shipCode.c_str(), iter->bpv, iter->yearFirstAvail, 
				iter->yearLastAvail);
		SHIP_LIST_ITER_END
		}
		debug_fprintf(fp, "\n");

		debug_fprintf(fp, "\nFighters Available:\n\n");
		debug_fprintf(fp, "Code                  BPV  YFA YLA\n");
#endif

		// take care of unwanted or unavailable fighters/PFs
		// the shiplist's YLA is in SFB year. Compensate by 164 years.
		SHIP_LIST_ITER(fighter_sl)
//			if ((iter->yearLastAvail - 164 < latestShip_first_available)
//				|| (iter->yearFirstAvail > fighter_date_end)) 
			if (
				(!
				((iter->yearLastAvail - 164 < latestShip_first_available)
				|| (iter->yearFirstAvail > selected_era_end_year)) 
				) && (!(iter->bpv < 0)) && (!(iter->geoUI == "XSH"))
				)
			{
				carry_over_fighter_sl.push_back(*iter);
				debug_fprintf(fp, "%-20s %4d %4d %3d\n",
					iter->shipCode.c_str(), iter->bpv, iter->yearFirstAvail, 
					iter->yearLastAvail);
			}
		SHIP_LIST_ITER_END

		debug_fprintf(fp, "\n");
	}
	else if (current_team == kTeam8)
	{
		debug_fprintf(fp, 
			"Second Enemy Team (kTeam8): continuing where kTeam7 left off...\n");
	}
	else if (current_team == kTeam9)
	{
		debug_fprintf(fp, 
			"Third Enemy Team (kTeam9): continuing where kTeam8 left off...\n");
	}
	else if (current_team == kTeam10)
	{
		debug_fprintf(fp, 
			"Fourth Enemy Team (kTeam10): continuing where kTeam9 left off...\n");
	}


	//
	// This code is executed for all of the enemy teams.
	// When the first team has MAX_ASSIGN ships, it spills over into
	// the next team.  When it runs out of ships selection is over.
	// We do this because if you put more than 8 ships on an enemy team
	// the engine seems to die.
	//


	if (bpv_to_assign >= cheapest_ship && carry_over_sl.size())
	{
		Ship		s;

		debug_fprintf(fp, "BPV left to assign: %d\n", bpv_to_assign);

		if (current_team == kTeam7)
		{
			// define classes and shiplists for each hull type
			/*
			std::list< eClassTypes >	mon_classes;
			std::list< eClassTypes >	bb_classes;
			std::list< eClassTypes >	dn_classes;
			std::list< eClassTypes >	cv_classes;
			std::list< eClassTypes >	bch_classes;
			std::list< eClassTypes >	ca_classes;
			std::list< eClassTypes >	nca_classes;
			std::list< eClassTypes >	cl_classes;
			std::list< eClassTypes >	wd_classes;
			std::list< eClassTypes >	dd_classes;
			std::list< eClassTypes >	ff_classes;

			// prepare each shiplist by hulltype
			if (gCoopData.mIsShipClassSelected(kClassFrigate))				{ ff_classes.push_back(kClassFrigate); }
			if (gCoopData.mIsShipClassSelected(kClassDestroyer))			{ dd_classes.push_back(kClassDestroyer); }
			if (gCoopData.mIsShipClassSelected(kClassWarDestroyer))			{ wd_classes.push_back(kClassWarDestroyer); }
			if (gCoopData.mIsShipClassSelected(kClassLightCruiser))			{ cl_classes.push_back(kClassLightCruiser); }
			if (gCoopData.mIsShipClassSelected(kClassHeavyCruiser))			{ ca_classes.push_back(kClassHeavyCruiser); }
			if (gCoopData.mIsShipClassSelected(kClassNewHeavyCruiser))		{ nca_classes.push_back(kClassNewHeavyCruiser); }
			if (gCoopData.mIsShipClassSelected(kClassHeavyBattlecruiser))	{ bch_classes.push_back(kClassHeavyBattlecruiser); }
			if (gCoopData.mIsShipClassSelected(kClassCarrier))				{ cv_classes.push_back(kClassCarrier); }
			if (gCoopData.mIsShipClassSelected(kClassDreadnought))			{ dn_classes.push_back(kClassDreadnought); }
			if (gCoopData.mIsShipClassSelected(kClassBattleship))			{ bb_classes.push_back(kClassBattleship); }
			if (gCoopData.mIsRaceSelected(kMonster))						{ mon_classes.push_back(kClassMonster);	}
			*/
		

			debug_fprintf(fp, "Preparing HullClass Subsets (API call: HullClassSubsets)\n");
			carry_over_sl.HullClassSubsets(&mon_sl, &bb_sl, &dn_sl, &cv_sl, &bch_sl, 
				&nca_sl, &ca_sl, &cl_sl, &wd_sl, &dd_sl, &ff_sl);

			debug_fprintf(fp, "DONE: HullClass Subsets (API call: HullClassSubsets)\n");
		} 
		/*
		else if (
			(current_team == kTeam8) || (current_team == kTeam9) || (current_team == kTeam10)
			)
		{
		}
		*/


		// populate a linked list of all the hulltype-based shiplists.
		hulltype_linklist *current_hulltype_sl;
		hulltype_linklist *previous_hulltype_sl;

	// prepare linklists only once.
	if (current_team == kTeam7)
	{
		debug_fprintf(fp, "\nPopulating link list\n");
		hulltype_sl_list = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
		current_hulltype_sl = hulltype_sl_list;
	
		if (mon_sl.size()) 
		{
			debug_fprintf(fp, "preparing Monsters node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			if(gCoopData.mIsRaceRandom())
				current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pMON);
			else
				current_hulltype_sl->oddsInPercent = 100;
			current_hulltype_sl->shiplistbyHull = &mon_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(mon_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (bb_sl.size()) 
		{
			debug_fprintf(fp, "preparing BB node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pBB);
			current_hulltype_sl->shiplistbyHull = &bb_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(bb_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (dn_sl.size()) 
		{
			debug_fprintf(fp, "preparing DN node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pDN);
			current_hulltype_sl->shiplistbyHull = &dn_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(dn_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (cv_sl.size()) 
		{
			debug_fprintf(fp, "preparing CV node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pCV);
			current_hulltype_sl->shiplistbyHull = &cv_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(cv_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (bch_sl.size()) 
		{
			debug_fprintf(fp, "preparing BCH node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pBCH);
			current_hulltype_sl->shiplistbyHull = &bch_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(bch_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (ca_sl.size()) 
		{
			debug_fprintf(fp, "preparing CA node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pCA);
			current_hulltype_sl->shiplistbyHull = &ca_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(ca_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (nca_sl.size()) 
		{
			debug_fprintf(fp, "preparing NCA node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pNCA); 
			current_hulltype_sl->shiplistbyHull = &nca_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(nca_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (cl_sl.size()) 
		{
			debug_fprintf(fp, "preparing CL node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pCL);
			current_hulltype_sl->shiplistbyHull = &cl_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(cl_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (wd_sl.size()) 
		{
			debug_fprintf(fp, "preparing WarDestroyer node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pDW);
			current_hulltype_sl->shiplistbyHull = &wd_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(wd_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (dd_sl.size()) 
		{
			debug_fprintf(fp, "preparing DD node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pDD);
			current_hulltype_sl->shiplistbyHull = &dd_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;
			current_hulltype_sl = current_hulltype_sl->Next;

			SHIP_LIST_ITER(dd_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}
		if (ff_sl.size()) 
		{
			debug_fprintf(fp, "preparing FF node\n");

			current_hulltype_sl->BPVtoMatch = 0;
			current_hulltype_sl->oddsInPercent = gCoopData.mGetShipOdds(pFF);
			current_hulltype_sl->shiplistbyHull = &ff_sl;
			current_hulltype_sl->Next = (hulltype_linklist *)malloc(sizeof(struct hulltype_linklist));
			previous_hulltype_sl = current_hulltype_sl;

			SHIP_LIST_ITER(ff_sl)
			if (! current_hulltype_sl->BPVtoMatch) 
			{
				current_hulltype_sl->BPVtoMatch = iter->bpv;
			} else {
				if (iter->bpv < current_hulltype_sl->BPVtoMatch)
					current_hulltype_sl->BPVtoMatch = iter->bpv;
			}
			SHIP_LIST_ITER_END
		}

		// there will always be at least one extra malloc 
		current_hulltype_sl = previous_hulltype_sl;
		free(current_hulltype_sl->Next);
		current_hulltype_sl->Next = hulltype_sl_list;
	}

		// prepare pointer to first item.
		current_hulltype_sl = hulltype_sl_list;
		
		debug_fprintf(fp, "Linklist populated. Entering shipselection loop\n");

		int numShipsPerTeam = 1;
	
		for(; 
			bpv_to_assign >= cheapest_ship && 
			numShips <= MAX_SHIPS &&
			numShipsPerTeam <= MAX_ASSIGN &&
			carry_over_sl.size();
		  ) 
		{
			if (bpv_to_assign >= current_hulltype_sl->BPVtoMatch &&
				(*(current_hulltype_sl->shiplistbyHull)).size())
			{
				/*
		  	  // Removing ships whose BPV are higher than bpv_to_assign
				if (bpv_to_assign < 90) {
					debug_fprintf(fp, 
						"Removing ships whose BPV are higher than bpv_to_assign.\n");

					(*(current_hulltype_sl->shiplistbyHull)).RemoveHigherThanBPV(bpv_to_assign);
					carry_over_sl.RemoveHigherThanBPV(bpv_to_assign);
					debug_fprintf(fp, 
						"DONE: Removing ships whose BPV are higher than bpv_to_assign.\n");
				}
				*/


			  // better to have a separate test. Faster due to less
			  // random number generation if second test if first fails
			  if (fMissionInfo->mRandomInt32(0, 100) < current_hulltype_sl->oddsInPercent)
			  {
				int result = (*(current_hulltype_sl->shiplistbyHull)).SelectRandomShip(s);
				int32 tmpval = bpv_to_assign - s.bpv;

				if (tmpval < 0)
				{
					// Not enough BPV left for this ship.
					// so remove from list and pick again from next hull category.
					debug_fprintf(fp, 
						"%s: costs too much. Remove from list and try with next type in linklist\n",
						s.shipCode.c_str());

					(*(current_hulltype_sl->shiplistbyHull)).Remove(s.shipCode);
					carry_over_sl.Remove(s.shipCode);


				} else {
					bpv_to_assign -= s.bpv;

					debug_fprintf(fp, 
						"Creating enemy ship #%d (Team %d/Ship %d) '%s' value %d\n",
						numShips, mGetTeam() + 1, numShipsPerTeam, 
						s.shipCode.c_str(), s.bpv);

					numShips++;  // 32 ships max supported by game
					numShipsPerTeam++;  // 8 ships max per team

					// moved to AFTER fleet is created.
					// Fixes problems.
					selected_enemy_sl.push_back(s);

					mCreateShip( typeid( tShipInfo ),
						s.shipCode.c_str(),
						kStartPosition_M + (int )mGetTeam()-6,
						kNoMetaShipID,
						-1, 0, 0, -1, -1, NULL, 
						static_cast< eShipOptions >(kDefaultShipOptions | kCanBeCarrier | kDeathUponCapture) 
					);
				}
			  }
			}
			// Next hulltype in linked list.
			// Note that this linklist loops around.
			current_hulltype_sl = current_hulltype_sl->Next;
		}


		// recursive linklist freeing
		// hulltype_linklist *hulltype_sl_list;
		// only after last team
		if (current_team == kTeam10) 
		{
			debug_fprintf(fp, "Freeing Linked List\n");

			free_linklist(hulltype_sl_list, hulltype_sl_list);
			free(hulltype_sl_list);
		}
	}

	debug_fprintf(fp, "Enemy setup complete.\n");
	debug_fprintf(fp, "Enemy remaining BPV: %d\n", bpv_to_assign);
}

void tEnemyTeam::mTeamSetupComplete()
{
	tTeamInfo::mTeamSetupComplete();
}


void tEnemyTeam::free_linklist(hulltype_linklist *list, hulltype_linklist *first)
{
	if (list->Next != first) {
		free_linklist(list->Next, first);
	} else {
		list->Next = NULL;
		return;
	}
	free(list->Next);
}


