/********************************************************************************************************\

	File Name:	EnemyTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/


#include "EnemyTeam.h"
#include "EnemyTeamBaseState.h"
#include "worldconditions.h"
#include "CoopCustomizer Common Files/CoopData.h"
#include "scriptinterface.h"
#include "Mul_CoopAce.h"
#include "CommonDefs.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.

*/



void tEnemyTeamBaseState::mEndState()
{}

void tEnemyTeamBaseState::mOnDeath( tShipInfo* ship )
{
	if (!gScriptInterface.mIsHost())
		return;

	if ( !ship )
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;	

#ifdef DEBUGMODE
	FILE *fp = fopen("C:\\Coop.txt", "a");
	debug_fprintf(fp, "enemy ship (%s) destroyed.\n", ship->mGetClassName());
#endif
	ship->mSetShipStatus(kShipDestroyed);

	mSetTimedInterval(kTimerSecondsSet, eCheckEnd, 1.0f);
#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tEnemyTeamBaseState::mOnCapture( tShipInfo* ship )
{
	if (!gScriptInterface.mIsHost())
		return;
	if ( !ship )
		return;
	if (ship->mGetOriginalTeam() != fChildActor->mGetTeam())
		return;	

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	/*
	int32 shipID = ship->mGetShipID();
	int shipentry = 0;

	for (int i = 0; i < shipcount; i++) {
		if (shipID == all_ships_IDs[i])
		{
			shipentry = i;
			break;
		}
	}

	//// Original capture code:
	//ship->mClearCommands();
	//
	//if (all_ships_teams[shipentry] >= (int32)kTeam7)
	//{
	//	fprintf(fp, "enemy ship (%s) captured.\n", ship->mGetClassName());
	//	fflush(fp);
	//}
	*/

	// Added SD code:
	// kDeathUponCapture does NOT work if there's been a host switch.
	ship->mSetShipMaxSpeed(0);

#ifdef OP
	ship->mSetShipImmediateSpeed(0);
#else
	ship->mSetShipThrust(0);
#endif

	ship->mDestroyShip();

	debug_fprintf(fp, "captured ship (%s) self-destructed.\n", 
		ship->mGetClassName());

	// clintk's capture code from forums:
	// http://208.57.228.3/ubb/Forum12/HTML/000520.html

	mSetTimedInterval(kTimerSecondsSet, eCheckEnd, 1.0f);
#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tEnemyTeamBaseState::mOnDisengaged( tShipInfo* ship )
{
	if (!gScriptInterface.mIsHost())
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;	

#ifdef DEBUGMODE
	FILE *fp = fopen("C:\\Coop.txt", "a");
	debug_fprintf(fp, "enemy ship (%s) disengaged.\n", ship->mGetClassName());
#endif

	ship->mSetShipStatus(kShipDisengaged);
	mSetTimedInterval(kTimerSecondsSet, eCheckEnd, 1.0f);

#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tEnemyTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tEnemyTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tEnemyTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}
void tEnemyTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{}
void tEnemyTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tEnemyTeamBaseState::mOnFlashingButtonPressed()
{}
void tEnemyTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{}
void tEnemyTeamBaseState::mOnChangingShip( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tEnemyTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{
/*
    // Capture intelligence check.
	if (!gScriptInterface.mIsHost())
		return;
	if (fChildActor->mGetTeam() != ship->mGetTeam())
		return;	

	FILE *fp = fopen("C:\\Coop.txt", "a");

	if ((type == kShipCurrentValue) && (system == kTransporterHardPoint)){
		fprintf(fp, "(Capture check) TransPorters: %s -> %d\n", ship->mGetClassName(), value);

		int32 shipID = ship->mGetShipID();
		int shipentry = 0;

		for (int i = 0; i < shipcount; i++) {
			if (shipID == all_ships_IDs[i])
			{
				shipentry = i;
				break;
			}
		}

		if (shipentry) {
			all_ships_state[shipentry] = eGotTransporters;
			all_ships_transporters[shipentry] = value;
			// Good, we have this info. Now fetch current # marines
			ship->mGetAmountOfStore( kMarines );
		}
	}
	fclose(fp);
*/
}
void tEnemyTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tEnemyTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}

void tEnemyTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tEnemyTeamBaseState::mOnHostChanged()
{
	if (!gScriptInterface.mIsHost())
		return;

	// Run only once.
	if (fChildActor->mGetTeam() != kTeam7)
		return;	
	gWC.mCheckForGameEnd();

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
	debug_fprintf(fp, "Host changed.\n");
    debug_fprintf(fp, "Populating needed shiplists.\n");
#endif
	if (!(selected_enemy_sl.size())) 
	{
		Populate_selected_enemy_sl();
	}

	/* We don't (yet?) need the fighterlist. Waste of time to load.
	debug_fprintf(fp, "Populating needed fighterlist.\n");
	if (!(carry_over_fighter_sl.size()))
	{
		Populate_carry_over_fighter_sl();
	}
	*/

	debug_fprintf(fp, "Resetting AI control. Prevents zombie AI ships after host left.\n");
	// Zombie AI fix.
	for (int l = (int)kTeam7; l <= (int)kTeam10; l++)
	{
		eTeamID team = (eTeamID)l;
		
		tTeamInfo *ti = gScriptInterface.mMission()->mGetTeamHandle((eTeamID)team);
		if (ti)
		{
			int32 i = ti->mGetShipsInPlayCount();		// can be -1... odd!

			//
			// If ship captured, must check current team
			//

			for (int k = 0; k < i; k++)
			{
				tShipInfo *	tsi = (tShipInfo *)ti->mGetThisShip(k+1);
				int32 shipTeam = tsi->mGetTeam();

#ifdef OP
				// With a new host, carriers were launching new fighters.
				RemoveFightersFromShip(tsi);
#endif

				// make sure the AI is turned on.
				tsi->mSetShipAIControlled(kTrue);
				tsi->mClearCommands();

				debug_fprintf(fp, "Ship (%s) reset to be AI controlled.\n", 
					tsi->mGetClassName());
			}
		}
	}

	// Initiate the eCheckForCapture timer.
	// mSetTimedInterval(kTimerSecondsSet, eCheckForCapture, 1.0f);
#ifdef DEBUGMODE
	fclose(fp);
#endif
}
void tEnemyTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tEnemyTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}

static char * sCvIntel(int32u i)
{
	switch (i)
	{
		case 0: return "Captain";
		case 1: return "Commodore";
		case 2: return "Admiral";
		default:return "Unknown Value";
	}
}

int32 pre_pass_bpv;
void tEnemyTeamBaseState::mOnTimedInterval( int32 timerID )
{
	// not host, doesn't matter
	if (!gScriptInterface.mIsHost())
		return;
	if (fChildActor->mGetTeam() < kTeam7)
		return;

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	eTeamID current_team = fChildActor->mGetTeam();
	if ((timerID == eCreateEnemyItems) && (current_team == kTeam7)) {
		bpv_to_assign += extras_bpv_to_assign * gCoopData.mGetEnemyBPVRatio(); 
	}
	
	if (timerID == eStartupMsg)
	{
		char buf[512];
#ifdef OP
		sprintf(buf, "MagnumMan's Co-op Ace 3.21 for SFC:OP\n(C) 2002 James E. King, III,\nmodified by Luc Lanthier and Yves Nantel\n\nDifficulty Multiplier: %1.2f (%s Level)\nEnemy Race: %s\0", gCoopData.mGetEnemyBPVRatio(),
			sCvIntel(gCoopData.mGetAIIntel()), (gCoopData.mIsRaceRandom() ? "Host-Selected Races" : "Single Random Race"));
#else
		sprintf(buf, "MagnumMan's Co-op Ace 3.21 for SFC:EAW\n(C) 2002 James E. King, III,\nmodified by Luc Lanthier and Yves Nantel\n\nDifficulty Multiplier: %1.2f (%s Level)\nEnemy Race: %s\0", gCoopData.mGetEnemyBPVRatio(),
			sCvIntel(gCoopData.mGetAIIntel()), (gCoopData.mIsRaceRandom() ? "Host-Selected Races" : "Single Random Race"));
#endif

		debug_fprintf(fp, "Sending mission credits to all teams.\n");
		for (int team = (int)kTeam1; team <= (int)kTeam6; team++) {
			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
			if (ti)
			{
				int32 i = ti->mGetShipsInPlayCount();		// can be -1... odd!
				if (i >= 0) {
					fMissionInfo->mDisplayMessage((eTeamID)team, buf);
				}
			}
		}
		mSetTimedInterval(kTimerSecondsSet, eCheckEnd, 1.0f);
	}

	// Check to see if the timer was needed to be acted upon.
	if ((timerID == eTimerWait) && (current_team == kTeam7))
	{
		if ((AlertStateChanged) && (DelayComplete))
		{
			mSetTimedInterval(kTimerSecondsSet, eSetupShips, 3.0f);
		} else {
			mSetTimedInterval(kTimerSecondsSet, eTimerWait, 0.5f);
		}
	}

	if ((timerID == eSetupShips) && (current_team >= kTeam7))
	{
		// Run only once per pass.
		if (current_team < kTeam7)
			return;

		// Set up the enemy fighters and loadouts
		// More damned bugs: We can set fighters per ship by #bays * #fighters/bay only
		// no different types per bay.

#ifdef OP
		debug_fprintf(fp, "LOADOUTS STEP 3: enemy timerID: eSetupShips\n");
		if (!gCoopData.mGet_upgrade_AI_fighters()) 
		{
			debug_fprintf(fp, "*** Fighter loadouts skipped, as per customizer.\n");
		}

		tTeamInfo *ti = fMissionInfo->mGetTeamHandle(current_team);
		if (ti && gCoopData.mGet_upgrade_AI_fighters())
		{
			tShipIterator tsi = ti->mGetFirstShip();
			for (;;)
			{
				Ship aShip;
				tShipInfo *si = *tsi;
				if (!si)
					break;

				selected_enemy_sl.Find(si->mGetClassName(), aShip);
			
				debug_fprintf(fp, "preparing enemy fighters for: (%s).\n", 
					si->mGetClassName());

				// check for fighters.
				// up to 4 fighter bays
				int num_bays = 0;
				int32 count = 0;
				int32 total_num_fighters = 0;
				Ship selected_fighter;
				BaseShip fighter;
				ShipList tmpfighter_sl;
				iTruth found_fighter = FALSE;

				for (int k = 0; k < 4; k++)
				{
					// assume all fighters are the same in the ship
					// load only the first one.
					if (! found_fighter) {
						fighter = aShip.fighters[k].first;
						found_fighter = TRUE;
					}
					count = aShip.fighters[k].second;
					if (count)
					{
						num_bays++;
						total_num_fighters += count;
					}
				}

				debug_fprintf(fp, "total_num_fighters: %d\n", total_num_fighters);

				if (total_num_fighters)
				{
					// We have fighters!
					// print out: fighter.shipCode
					debug_fprintf(fp, "We have fighters! Bays: %d Count: %d  Type: %s\n",
						num_bays, total_num_fighters, fighter.shipCode.c_str());

					debug_fprintf(fp, "Removing fighter selection which are less than %d BPV.\n",
						fighter.bpv);

					SHIP_LIST_ITER(carry_over_fighter_sl)
						if (
							(iter->race == aShip.race)
							&& (iter->bpv >= fighter.bpv)
						) {
							tmpfighter_sl.push_back(*iter);
						}
					SHIP_LIST_ITER_END

					if (tmpfighter_sl.size())
						tmpfighter_sl.SelectRandomShip(selected_fighter);

						int32 avg_fighters = total_num_fighters / num_bays;
					if (total_num_fighters % num_bays)
						avg_fighters++;
						// print out: fighter.shipCode

					debug_fprintf(fp, "New Fighter type: Bays: %d   #/bay: %d  Type: %s\n",
						num_bays, avg_fighters, selected_fighter.shipCode.c_str());

					si->mSetFighters(num_bays, 
						avg_fighters, selected_fighter.shipCode);
				}
				tsi++;
			}
		}
#endif  // OP


		if (current_team == kTeam7)
			mSetTimedInterval(kTimerSecondsSet, eStartupMsg, 7.0f);

		// Now to equip the ships. Delay needed?
		if (current_team == kTeam7)	{
			mSetTimedInterval(kTimerSecondsSet, eCreateEnemyItems, 2.0f);
			// mSetTimedInterval(kTimerSecondsSet, eCheckForCapture, 8.0f);
		}
		mSetTimedInterval(kTimerSecondsSet, eCheckEnemyItems, 4.0f);
	} 

	if (
		(timerID == eCreateEnemyItems) 
		&& (current_team == kTeam7)  // run only once.
		)
	{
		// Adding items to store won't work. Fine!
		// STEP 1: 
		//  prepare 2D array.

		debug_fprintf(fp, "LOADOUTS STEP 4: enemy timerID: eCreateEnemyItems\n");
		int32 loadouts[3][32];
		for (int x=0; x<3; x++)
			for (int y=0; y<32; y++)
				loadouts[x][y] = 0;

		int32 numships2 = 0;
			//////////////////////////////////////////////////////////////
		// STEP 2:
		//  Populate 2D array with loadouts.
		for (int l = (int)kTeam7; l <= (int)kTeam10; l++)
		{
			eTeamID team = (eTeamID)l;
		
			if (team == kTeam7)
				pre_pass_bpv = bpv_to_assign;

			// and HERE we start adding the items to enemy ships and 
			// reduce the value of bpv_to_assign accordingly.
			debug_fprintf(fp, "Creating Items for enemy. Team: %d  bpv_to_assign: %d\n",
				(int )team, bpv_to_assign);

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle(team);
			if (ti)
			{
				tShipIterator tsi = ti->mGetFirstShip();
				for (;;)
				{
					tShipInfo *si = *tsi;
					if (!si)
						break;

					numships2++;

					if ((bpv_to_assign > 0) && (si->mGetRace() != kMonster))
					{
						int32 itemSelected = fMissionInfo->mRandomInt32(0, kLastItem-1);

						if (itemSelected == ktBombItem) {
							bpv_to_assign -= ktBombItemValue;
							debug_fprintf(fp, "ship: (%s)  tbomb++  bpv_to_assign: %d\n", 
								si->mGetClassName(), bpv_to_assign);

							loadouts[itemSelected][numships2-1]++;
						} else if (itemSelected == kMarineItem) {
							bpv_to_assign -= kMarineItemValue;
							debug_fprintf(fp, "ship: (%s)  Marine++  bpv_to_assign: %d\n", 
								si->mGetClassName(), bpv_to_assign);

							loadouts[itemSelected][numships2-1]++;
						} else if (itemSelected == kSparePartItem) {
							bpv_to_assign -= kSparePartItemValue;

							debug_fprintf(fp, "ship: (%s)  SparePart++  bpv_to_assign: %d\n", 
								si->mGetClassName(), bpv_to_assign);

							loadouts[itemSelected][numships2-1]++;
						}
					}
					tsi++;
				}
			}

			if (
				(team == kTeam10) 
				&& (pre_pass_bpv != bpv_to_assign)
				&& (bpv_to_assign > 0)
				)
			{
				debug_fprintf(fp, "We're going for another loop. ppb: %d  bta: %d\n", 
					pre_pass_bpv, bpv_to_assign);

				// we're gonna l++, so do a -1.
				l = (int )kTeam7 -1;
				numships2 = 0;
			}
		}
		//////////////////////////////////////////////////////////////////
		// STEP 3:
		//  Set #items on each ship to values stored in loadouts
		int32 numships3 = 0;
		for (int m = (int)kTeam7; m <= (int)kTeam10; m++)
		{
			eTeamID team = (eTeamID)m;

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle(team);
			if (ti)
			{
				tShipIterator tsi = ti->mGetFirstShip();
				for (;;)
				{
					tShipInfo *si = *tsi;
					if (!si)
						break;

					numships3++;

					if (si->mGetRace() != kMonster) {
						si->mSetItemInStores(kSmall_Mines, kStandard, 
							loadouts[ktBombItem][numships3-1]);
						si->mSetItemInStores(kMarines, kStandard, 
							loadouts[kMarineItem][numships3-1]);
						si->mSetItemInStores(kSpareParts, kStandard, 
							loadouts[kSparePartItem][numships3-1]);
					}
					tsi++;
				}
			}
		}
	}

	if (timerID == eCheckEnd)
	{
		gWC.mCheckForGameEnd();
	}

	// Run once for all enemy teams
	if ((timerID == eCheckEnemyItems) && (current_team == kTeam7))
	{
		debug_fprintf(fp, "LOADOUTS STEP 5: enemy timerID: eCheckEnemyItems\n");
		for (int l = (int)kTeam7; l <= (int)kTeam10; l++)
		{
			eTeamID team = (eTeamID)l;
			tTeamInfo *ti = fMissionInfo->mGetTeamHandle(team);
			if (ti)
			{
				tShipIterator tsi = ti->mGetFirstShip();
				for (;;)
				{
					tShipInfo *si = *tsi;
					if (!si)
						break;

					si->mGetAmountOfStore( kSpareParts );
					si->mGetAmountOfStore( kMarines );
					si->mGetAmountOfStore( kSmall_Mines );

					// Sets ship to use Type IVs.; doesn't work
					// si->mGetAmountOfStore( kMissilesTypeI );

					tsi++;
				}
			}
		}
		// Last enemy team, set a little delay and prevent some 
		// startup conditions to be met later.
		mSetTimedInterval(kTimerSecondsSet, eSetCompletedStartup, 1.0f);
	} 

	/* AI change of behaviour didn't work.
	if ((timerID == eCheckForCapture) && (current_team >= kTeam7))
	{
		tTeamInfo *ti = fMissionInfo->mGetTeamHandle(current_team);
		if (ti)
		{
			tShipIterator tsi = ti->mGetFirstShip();
			for (;;)
			{
				Ship aShip;
				tShipInfo *si = *tsi;
				if (!si)
					break;

				si->mGetHealthOfSystem(kTransporterHardPoint);
				tsi++;
			}
		}
//		if (current_team == kTeam7) {
//			mSetTimedInterval(kTimerTurnsSet, eCheckForCapture, 2.0f);
//		}
	}
	*/

	if (
		(timerID == eSetCompletedStartup) 
		&& (current_team == kTeam7)
		&& (completed_startup == 0)
		)
	{
		completed_startup = 1;
		debug_fprintf(fp, "STARTUP COMPLETED.\n");
	}
#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tEnemyTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{
	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	if (fChildActor->mGetTeam() < kTeam7)
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;	

	if (!completed_startup)	{
		PrepareEnemyLoadout(ship, type, item, subItem, value);
	}
	else 
	{
#ifdef PROBE_TWEAK
		// probing for info.
		ProbedEnemyLoadout(ship, type, item, subItem, value);
#endif


	/*
	// This AI logic did NOTHING.
		int32 shipID = ship->mGetShipID();
		int shipentry = 0;

		for (int i = 0; i < shipcount; i++) {
			if (shipID == all_ships_IDs[i])
			{
				shipentry = i;
				break;
			}
		}

		if (all_ships_state[shipentry] == eGotTransporters) {
			if ((type == kShipCurrentValue) && (item == kMarines))
			{
				fprintf(fp,"(capture check) Marines: %s -> %d\n", ship->mGetClassName(), value);
				if (shipentry) {
					all_ships_state[shipentry] = eGotOwnMarines;
					all_ships_marines[shipentry] = value;

					// get current target's shipID and store it.
					int32 target_ShipID = ship->mGetShipTargetID();
					all_ships_targets[shipentry] = target_ShipID;

					// Find target's tShipInfo entry by finding its ShipID
					eTeamID target_team;
					for (int i = 0; i < shipcount; i++) {
						if (target_ShipID == all_ships_IDs[i])
						{
							target_team = (eTeamID) all_ships_teams[i];
							break;
						}
					}

					// Ask for number of marines.
					tTeamInfo *ti = 
						fMissionInfo->mGetTeamHandle((eTeamID)target_team);
					if (ti)
					{
						int32 i = ti->mGetShipsInPlayCount();
						for (int32 k = 0; k < i; k++)
						{
							tShipInfo *si = (tShipInfo *)ti->mGetThisShip(k+1);
							if (!si)
								break;
							if (si->mGetShipID() == target_ShipID) 
							{
								si->mGetAmountOfStore( kMarines );
							}
						}
					}
				}
			}
		}
	*/

	}

#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tEnemyTeamBaseState::mBeginState()
{
	// those who are not host, do not continue.
	// if we have already previously done startup, return. (host switch).
	if (!gScriptInterface.mIsHost()) {
		completed_startup = 1;
		return;
	}

	eTeamID current_team = fChildActor->mGetTeam();
	if (current_team < kTeam7)
		return;

	mSetTimedInterval(kTimerSecondsSet, eTimerWait, 15.0f);
}



// Custom Functions
//
void tEnemyTeamBaseState::PrepareEnemyLoadout(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{
	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	if (!gScriptInterface.mIsHost()) {
#ifdef DEBUGMODE
		debug_fprintf(fp, "Non-host, what are you doing here? Trying to probe but completed_startup isn't set?\n");
		fclose(fp);
#endif
		return;
	}

	Ship aShip;
	if ((type == kShipCurrentValue) && (item == kMarines))
	{
		selected_enemy_sl.Find(ship->mGetClassName(), aShip);
		if ((aShip.race != kMonster) || (aShip.marinesMax > 0 )) {
			debug_fprintf(fp, "ship (%s) normally carries %d marines.\n", 
				ship->mGetClassName(), aShip.marinesBase);

			int newval = value+aShip.marinesBase;
			if (newval > aShip.marinesMax)
				newval = aShip.marinesMax;

			debug_fprintf(fp, "It is now carrying %d marines.\n",	newval);

			ship->mSetItemInStores(item, subItem, newval);
		}
	}
	else if ((type == kShipCurrentValue) && (item == kSmall_Mines))
	{
		selected_enemy_sl.Find(ship->mGetClassName(), aShip);
		if ((aShip.race != kMonster) || (aShip.tBombsMax > 0 )){
			debug_fprintf(fp, "ship (%s) normally carries %d tBombs.\n", 
				ship->mGetClassName(), aShip.tBombsBase);

			int newval = value+aShip.tBombsBase;
			if (newval > aShip.tBombsMax)
				newval = aShip.tBombsMax;

			debug_fprintf(fp, "It is now carrying %d tBombs.\n", newval);
			ship->mSetItemInStores(item, subItem, newval);
		}
	}
	else if ((type == kShipCurrentValue) && (item == kSpareParts))
	{
		selected_enemy_sl.Find(ship->mGetClassName(), aShip);
		if (aShip.race != kMonster) {
			debug_fprintf(fp, "ship (%s) normally carries %d spareparts.\n", 
				ship->mGetClassName(), DamCon_to_SpareParts(aShip.damageControl));

			int newval = DamCon_to_SpareParts(aShip.damageControl)+value;
			if (newval > DamCon_to_SpareParts(aShip.damageControl)*5)
				newval = DamCon_to_SpareParts(aShip.damageControl)*5;

			debug_fprintf(fp, "It is now carrying %d spareparts.\n", newval);
			ship->mSetItemInStores(item, subItem, newval);
		}
	}

/*
	// Set drones to Type IV.
	else if (item == kMissilesTypeI)
	{
#ifdef DEBUGMODE
		fprintf(fp, "ship (%s) normally carries %d MissilesTypeI.\n", 
			ship->mGetClassName(), value);
#endif

		if (value > 0) {
			ship->mSetItemInStores(kMissilesTypeIV, subItem, value);
#ifdef DEBUGMODE
			fprintf(fp, "ship (%s) now carries %d MissilesTypeIV.\n", 
				ship->mGetClassName(), value);
#endif
		}
	} 
*/
	  
	else {
		debug_fprintf(fp, "Unknown item #%d called for ship (%s).\n", 
			item, ship->mGetClassName());
	}

#ifdef DEBUGMODE
	fclose(fp);
#endif
}

// The following are queues.
// 			ProbeTargetsShipID[n]
//			ProbeSendersShipID[n]

#ifdef PROBE_TWEAK
void tEnemyTeamBaseState::ProbedEnemyLoadout(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{
    FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	int32 SourceShipID = 0;

	for (int i = 0; i < 108; i++) {
		if (
			(ProbeTargetsShipID[i] != 0) 
			&& (ProbeSendersShipID[i] != 0)
			&& (ship->mGetShipID() == ProbeTargetsShipID[i])
		)
		{
			int32 ScanTargetID = ProbeTargetsShipID[i];
			int32 ScanSourceID = ProbeSendersShipID[i];
			ProbeTargetsShipID[i] = 0;
			ProbeSendersShipID[i] = 0;

			// Get the sender's teamID.
			eTeamID ProbingTeamID = fChildActor->mGetTeam();
			for (int k = (int)kTeam1; k <= (int)kTeam6; k++)
			{
				eTeamID team = (eTeamID)k;

				tTeamInfo *ti = fMissionInfo->mGetTeamHandle(team);
				if (ti)
				{
					tShipIterator tsi = ti->mGetFirstShip();
					for (;;)
					{
						tShipInfo *si = *tsi;
						if (!si)
							break;

						if (si->mGetShipID() == ScanSourceID)
						{
							ProbingTeamID = team;
							debug_fprintf(fp, "CONFIRM: it was ship %s in team %d who probed.\n", 
								si->mGetClassName(), k);
							ProbingTeamID = (eTeamID)k;
							SourceShipID = ScanSourceID;
							break;
						}

						tsi++;
					}
				}
			}

			// Order of printed-out text is spareparts, marines, tBombs.
			if ((type == kShipCurrentValue) && (item == kSpareParts))
			{
				debug_fprintf(fp, "Ship %s probed for spareparts: %d\n", 
					ship->mGetClassName(), value);

				// send message to probing unit player
				char buf[512];
				sprintf(buf, "\nProbed enemy ship (%s) has %d SpareParts.\n", 
					ship->mGetClassName(), value);
				fMissionInfo->mDisplayMessage(ProbingTeamID, buf, kScienceScriptMessage);

			} else if ((type == kShipCurrentValue) && (item == kMarines))
			{
				debug_fprintf(fp, "Ship %s probed for Marines: %d\n", 
					ship->mGetClassName(), value);

				// send message to probing unit player
				char buf[512];
				sprintf(buf, "It holds %d Marines.\n", value);
				fMissionInfo->mDisplayMessage(ProbingTeamID, buf, kScienceScriptMessage);
			}
			else if ((type == kShipCurrentValue) && (item == kSmall_Mines))
			{
				debug_fprintf(fp, "Ship %s probed for tBombs: %d\n", 
					ship->mGetClassName(), value);

				// send message to probing unit player
				char buf[512];
				sprintf(buf, "It also seems to have %d transporter bombs.\n", 
					value);

				fMissionInfo->mDisplayMessage(ProbingTeamID, buf, kScienceScriptMessage);
			}

			break;
		}
	}

#ifdef DEBUGMODE
	fclose(fp);
#endif
}
#endif


