/********************************************************************************************************\

	File Name:	HumanTeamBaseState.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "HumanTeam.h"
#include "HumanTeamBaseState.h"
#include "EnemyTeamBaseState.h"
#include "worldconditions.h"
#include "CoopCustomizer Common Files/CoopData.h"
#include "Mul_CoopAce.h"
#include "CommonDefs.h"

/*

	HELPER NOTE:	There are so many things that can be done here depending upon which mission you are
					coding.  So rather than list out possible bodies for each of the events below, the
					signatures have been given to save time cutting and pasting.

*/


void tHumanTeamBaseState::mEndState()
{}
void tHumanTeamBaseState::mOnDeath( tShipInfo* ship )
{
	if ( !ship )
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;	

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
	debug_fprintf(fp, "player ship (%s) destroyed.\n", ship->mGetClassName());
#endif
	ship->mSetShipStatus(kShipDestroyed);

#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tHumanTeamBaseState::mOnCapture( tShipInfo* ship )
{
	/*
	// NOTE. This is for capturing someone ELSE's ship
	// use "ship->mGetOriginalTeam()".

	// Some failed attempt to allow captures in multiplayer.
	// leave commented out and let captured ships Self-Destruct

	if ( !ship )
		return;
	if (!gScriptInterface.mIsHost())
		return;

	tTeamInfo* capturedTeam = fMissionInfo->mGetTeamHandle(ship->mGetOriginalTeam());
	tTeamInfo* capturingTeam = fMissionInfo->mGetTeamHandle(ship->mGetTeam());

	int32 shipID = ship->mGetShipID();
	int shipentry = 0;

	for (int i = 0; i < shipcount; i++) {
		if (shipID == all_ships_IDs[i])
		{
			shipentry = i;
			break;
		}
	}

	FILE *fp = fopen("C:\\Coop.txt", "a");
	// If the following test passes, childactor is the CAPTURING team
	// CAPTURED would be "mGetOriginalTeam"..
	if (
		(ship->mGetTeam() == fChildActor->mGetTeam())
		&& (all_ships_teams[shipentry] >= (int32)kTeam7)
		)
	{
		fprintf(fp, "player/team %d captured ship (%s) originally from team %d.\n", 
			fChildActor->mGetTeam(), 
			ship->mGetClassName(),
			ship->mGetOriginalTeam());
		fflush(fp);
		

		// SFB says: 
		//   1/6 chance of self-destruct on capture attempt.
		//   2/6 chance, reroll next turn (not useful to us)
		//   3/6 chance of successful capture.
		// ... so.. 66% chance of successful capture sounds good? :)

		//if (fMissionInfo->mRandomInt32(0, 100) < 33) {
		if (0) {
			// sometimes, in games where a captured ship is captured and self-destructed
			// the game will end in failure. Maybe the "ghost" of the ship disengaged?
			// We won't have any of that.
			ship->mSetShipMaxSpeed(0);
			ship->mSetShipImmediateSpeed(0);
			ship->mDestroyShip();
			fprintf(fp, "captured ship (%s) self-destructed.\n", 
				ship->mGetClassName());
		} else {
			// update our own custom capture tracker.
			all_ships_teams[shipentry] = (int32)fChildActor->mGetTeam();

			RemoveFightersFromShip(ship);
			ship->mRemoveTargets();
			ship->mClearCommands();
		}
		fclose(fp);
	}
	*/
}

void tHumanTeamBaseState::mOnDisengaged( tShipInfo* ship )
{
	if ( !ship )
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;	

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
	debug_fprintf(fp, "player ship (%s) disengaged.\n", ship->mGetClassName());
	fclose(fp);
#endif
}
void tHumanTeamBaseState::mAtDistance( tShipInfo* ship )
{}
void tHumanTeamBaseState::mAtDestination( tShipInfo* ship )
{}
void tHumanTeamBaseState::mOnCommButtonPress( tShipInfo* ship, int32 buttonID, tShipInfo* targetShip )
{}
void tHumanTeamBaseState::mOnDamage( tShipInfo* damagedShip, tShipInfo* hostileShip, eDamageToShip damageType )
{}
void tHumanTeamBaseState::mOnDeepScan( tShipInfo* ship, iTruth deepScanOn )
{}

// Use this to detect when player1 (host) 
void tHumanTeamBaseState::mOnAlertStatusChange( tShipInfo* ship, eAlertLevels alertStatus )
{
	if (fChildActor->mGetTeam() >= kTeam7)
		return;
	eTeamID current_team = fChildActor->mGetTeam();
	if (ship->mGetTeam() != current_team)
		return;
	own_player_team = current_team;

	if (
		(! DelayComplete)
		&& (!completed_startup) 
		&& (!AlertStateChanged) 
		&& (alertStatus != kAlertGreen)
		) 
	{
		AlertStateChanged = 1;

		if (!gScriptInterface.mIsHost())
			return;
		mSetTimedInterval(kTimerSecondsSet, eStartDelay, 23.0f);
	}
}

// Will only run on machine where probe was launched.
// Seems like only host can make text msgs display, so this function
//   is good only from the host's.
void tHumanTeamBaseState::mOnProbeLaunch( tShipInfo* ship )
{
#ifdef PROBE_TWEAK
	if (fChildActor->mGetTeam() >= kTeam7)
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;	

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	int32 ScanTargetID = ship->mGetShipTargetID();
	// Find the target by checking all teams.
	// only enemy teams apply ;>
	for (int team = (int)kTeam7; team <= (int)kTeam10; team++)
	{
		tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
		if (ti)
		{
			tShipIterator tsi = ti->mGetFirstShip();
			for (;;)
			{
				tShipInfo *si = *tsi;
				if (!si)
					break;

				if ( ScanTargetID == si->mGetShipID() ) {
					debug_fprintf(fp, "team %d: probe launched by ship %s to ship %s\n", 
						ship->mGetTeam(), ship->mGetClassName(), 
						si->mGetClassName());

					// find a blank spot in the arrays of probe shipIDs 
					// and put in the values
					int done_prepping = 0;
					for (int i = 0; i < 108 && done_prepping < 3; i++) {
						if ((ProbeTargetsShipID[i] == 0) && (ProbeSendersShipID[i] == 0))
						{
							ProbeTargetsShipID[i] = ScanTargetID;
							ProbeSendersShipID[i] = ship->mGetShipID();
							if (done_prepping == 0) {
								si->mGetAmountOfStore( kSpareParts );
								done_prepping=1;
								debug_fprintf(fp, "fetching spareparts info\n");
							} else if (done_prepping == 1) {
								si->mGetAmountOfStore( kMarines );
								done_prepping=2;
								debug_fprintf(fp, "fetching marines info\n");
							} else if (done_prepping == 2) {
								si->mGetAmountOfStore( kSmall_Mines );
								done_prepping=3;
								debug_fprintf(fp, "fetching tBombs info\n");
							}
						}
					}
				}
				tsi++;
			}
		}
	}
#ifdef DEBUGMODE
	fclose(fp);
#endif
#endif
}

void tHumanTeamBaseState::mOnBeginOrbit( tShipInfo* ship )
{}
void tHumanTeamBaseState::mOnEndOrbit( tShipInfo* ship )
{}
void tHumanTeamBaseState::mOnBeginTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tHumanTeamBaseState::mOnEndTractor( tShipInfo* tractorTarget, tShipInfo* tractoringShip )
{}
void tHumanTeamBaseState::mOnFlashingButtonPressed()
{}
void tHumanTeamBaseState::mOnTransportedItem( tShipInfo* shipTo, tShipInfo* shipFrom, eScriptTransportItems item )
{
	// never got triggered?! Doesn't work.
	/*
	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif
	debug_fprintf(fp, "Transport occured. From %s to %s.\n", 
		shipFrom->mGetClassName(), shipTo->mGetClassName());
	if (item == kTransSpareParts)
	{
		debug_fprintf(fp, "SparePart Transported.\n");
	} 
	else 
	{
		debug_fprintf(fp, "Unknown item: %d\n", (int)item);
	}

#ifdef DEBUGMODE
	fclose(fp);
#endif
	*/
}
void tHumanTeamBaseState::mOnChangingShip( tShipInfo* ship )
{
	/* Only applies to captured ships, which we won't have any longer

	if (fChildActor->mGetTeam() >= kTeam7)
		return;
	if (ship->mGetTeam() != fChildActor->mGetTeam())
		return;

	FILE *fp = fopen("C:\\Coop.txt", "a");

	fprintf(fp, "Player %d switched ships.\n", (int)ship->mGetTeam());
	fflush(fp);

	// Fetch the currently controlled ship
	tTeamInfo *ti = fMissionInfo->mGetTeamHandle(ship->mGetTeam());

	tShipInfo *current_ship = NULL;
	Assert( ti );
	if ( ti )
		if ( ti->mIsTeamHumanControlled() )
			current_ship = ti->mCurrentlyControlledShip();

	if (! current_ship) {
		fprintf(fp, "Currently Controlled Ship data unavailable. Sorry..\n");
		fprintf(fp, "Returning to previous ship.\n");
		fflush(fp);

		// return to a properly set-up ship: the previous.
		ship->mSetShipPlayerControlled();
	}

	fclose(fp);
	*/
}
void tHumanTeamBaseState::mOnSpecialTransport( tShipInfo* ship )
{}
void tHumanTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, int32 value )
{}
void tHumanTeamBaseState::mOnShipSystemUpdate( tShipInfo* ship, eShipUpdateType type, eSystemHardPoint system, float percent )
{}
void tHumanTeamBaseState::mOnPlayerLeaving( int32 playerNumber, eTeamID TeamID )
{}
void tHumanTeamBaseState::mOnHostChanged()
{
	// Only once per hostchange
	if (fChildActor->mGetTeam() != own_player_team)
		return;
	int new_host_selected = 0;

	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
	debug_fprintf(fp, "Host Changed. Verifying Player teams.\n");
#endif


	char host_changed_msg[100];
	for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
	{
		tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
		if (ti)
		{
			// new lowest team member is the new host.
			/*
			if (!new_host_selected) {
				new_host_selected = 1;
				if (own_player_team == (eTeamID)team) {
					debug_fprintf(fp, "Setting THIS machine to become the new host.\n");
					gScriptInterface.mSetNewHost();
				}
			}
			*/

			if (gScriptInterface.mIsHost()) {
				int32 i = ti->mGetShipsInPlayCount();		// can be -1... odd!
				if (i >= 0) {
					debug_fprintf(fp, "Player/Team %d has %d ships left.\n", 
						team, i);
					sprintf(host_changed_msg, "Hey team %d! Host Changed.\0", team);
					fMissionInfo->mDisplayMessage((eTeamID)team, host_changed_msg);
				}
			}
		}
	}
	if (gScriptInterface.mIsHost())
		gWC.mCheckForGameEnd();

#ifdef DEBUGMODE
	fclose(fp);
#endif
}
void tHumanTeamBaseState::mOnStateChange( tActor* actor, tState* stateID, eStateEvent eventType )
{}
void tHumanTeamBaseState::mOnCommStateChange( tActor* actor, int32 state, eStateEvent eventType )
{}
void tHumanTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eScriptTransportItems subItem, int32 value )
{}


void tHumanTeamBaseState::mOnTimedInterval( int32 timerID )
{
	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	eTeamID current_team = fChildActor->mGetTeam();
	if (current_team >= kTeam7)
		return;

	if ((timerID == eStartDelay) && (current_team == kTeam1) && (!completed_startup))
	{
		debug_fprintf(fp, "Post-RedAlert StartDelay has passed. Setting up enemy ships.\n");
		mSetTimedInterval(kTimerSecondsSet, eCheckHumanLoadouts, 2.0f);
		// mSetTimedInterval(kTimerSecondsSet, ePrepareShipState, 0.5f);

		// With this set, the AIs can get set up too.
		DelayComplete=1;
	} 

	if ((timerID == eBeginWait) && (current_team == kTeam1))
	{
		debug_fprintf(fp, "eBeginWait has expired. Setting player to red alert.\n");
		tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)kTeam1);
		if ((ti) && (ti->mIsTeamHumanControlled()))
		{

			AlertStateChanged = 1;

			if (gScriptInterface.mIsHost()) {
				mSetTimedInterval(kTimerSecondsSet, eStartDelay, 23.0f);
			}

			tShipIterator tsi = ti->mGetFirstShip();
			tShipInfo *si = *tsi;
			if (si)
			{
				if (si->mGetAlertStatus() != kAlertRed)
				{
					si->mSetAlertStatus(kAlertRed);
				}
			}
		}
	}

	// All player machines should do this, in case host drops.
	/*
	if ((timerID == ePrepareShipState) && (current_team == kTeam1))
	{
		// Initialize arrays.
		for (int i = 0; i < 50; i++)
		{
			all_ships_IDs[i] = 0;
			// -1 = undefined team
			all_ships_teams[i] = -1;

			// following related to AI capturing humans code.
			all_ships_state[i] = eNothing;
			all_ships_transporters[i] = 0;
			all_ships_marines[i] = 0;
			all_ships_targets[i] = 0;
		}

		shipcount = 0;
		for (int team = (int)kTeam1; team <= (int)kTeam10; team++)
		{
			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
			if (ti)
			{
				int32 i = ti->mGetShipsInPlayCount();
				for (int32 k = 0; k < i; k++)
				{
					tShipInfo *si = (tShipInfo *)ti->mGetThisShip(k+1);
					if (!si)
						break;

					all_ships_IDs[shipcount] = si->mGetShipID();
					all_ships_teams[shipcount] = team;
					shipcount++;
				}
			}
		}
	}
	*/

	if ((timerID == eCheckHumanLoadouts) && (current_team == kTeam1) && (!completed_startup))
	{
		if (!gScriptInterface.mIsHost())
			return;
		debug_fprintf(fp, "LOADOUTS STEP 1: eCheckHumanLoadouts\n");

		for (int team = (int)kTeam1; team <= (int)kTeam6; team++)
		{
			// Setting drone loadouts does not work :(
			

			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
			if (ti)
			{
				int32 i = ti->mGetShipsInPlayCount();
				for (int32 k = 0; k < i; k++)
				{
					tShipInfo *si = (tShipInfo *)ti->mGetThisShip(k+1);
					if (!si)
						break;
					// For some reason, asking for a drone type will work for both
					// types I and IV.
					//si->mGetAmountOfStore( kMissilesTypeI );
	
					si->mGetAmountOfStore( kSpareParts );
					si->mGetAmountOfStore( kMarines );
					si->mGetAmountOfStore( kSmall_Mines );
				}
			}
		}
	}
#ifdef DEBUGMODE
	fclose(fp);
#endif
}

void tHumanTeamBaseState::mOnShipStoresUpdate(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{

	eTeamID current_team = fChildActor->mGetTeam();
	// AI teams, don't come here.
	if (current_team >= kTeam7)
		return;

	// STARTUP GETITEMS
	if ((!completed_startup) && ( current_team == ship->mGetTeam() )) {
		if (!gScriptInterface.mIsHost())
			return;
		StartupGetItem(ship, type, item, subItem, value);
	}
	// END STARTUP

	/*
	// Ship capture decision code
	// DOES NOT WORK: seems telling a ship to capture is done by 'tag' in multiplayer
	if (
		(completed_startup) 
		&& (type == kShipCurrentValue) 
		&& (item == kMarines)
		) 
	{
		if (!gScriptInterface.mIsHost())
			return;

		int32 shipID = ship->mGetShipID();
		int shipentry = 0;

		// Find out if this ship is being targetted. (and by whom)
		for (int i = 0; i < shipcount; i++) {
			if (shipID == all_ships_targets[i])
			{
				shipentry = i;
				break;
			}
		}
		if (all_ships_state[shipentry] == eGotOwnMarines) 
		{
			all_ships_state[shipentry] = eNothing;
			all_ships_targets[shipentry] = 0;

			// discover agressor's tShipInfo
			FILE *fp = fopen("C:\\Coop.txt", "a");

			// Ask for number of marines.
			tTeamInfo *ti = 
				fMissionInfo->mGetTeamHandle((eTeamID)all_ships_teams[shipentry]);
			if (ti)
			{
				int32 i = ti->mGetShipsInPlayCount();
				for (int32 k = 0; k < i; k++)
				{
					tShipInfo *si = (tShipInfo *)ti->mGetThisShip(k+1);
					if (!si)
						break;
					if (si->mGetShipID() == all_ships_IDs[shipentry]) 
					{
						// We have the information. 
						fprintf(fp, "(Capture Check) %s's target ship %s has %d marines.\n", 
							si->mGetClassName(), ship->mGetClassName(), value);
						eTargetGoals goal = 
								ti->mGetGoal(ship->mGetTeam());
						// kGoalIgnore,    //'I' or '-'
						// kGoalCripple,   //'C'
						// kGoalCapture,   //'X'
						// kGoalDestroy,   //'D'

						if (all_ships_transporters[shipentry] > value)
						{
							if (goal == kGoalCapture)
							{
								fprintf(fp, "Goal remains capture.\n");
							} else
							{
								fprintf(fp, "Setting %s's team's (%d) goal to capture\n", 
									si->mGetClassName(), (int)si->mGetTeam());
								ti->mSetRelations(
									static_cast<eTeamID>(ship->mGetTeam()), 
									kTeamAtWar, 
									kGoalCapture, 
									1.0
								);
								goal = ti->mGetGoal(ship->mGetTeam());

								if (goal == kGoalCapture) {
									fprintf(fp, "Goal successfully set to capture.\n");
								} else if (goal == kGoalDestroy) {
									fprintf(fp, "Goal still set to destroy. Bad.\n");
								}
							}
						} else {
							fprintf(fp, "Setting %s's team's (%d) goal to destroy\n", 
								si->mGetClassName(), (int)si->mGetTeam());
							eTargetGoals goal = 
								ti->mGetGoal(ship->mGetTeam());
							if (goal != kGoalDestroy) {
								ti->mSetRelations(
									static_cast<eTeamID>(ship->mGetTeam()), 
									kTeamAtWar, 
									kGoalDestroy, 
									1.0
								);
							}
						}
					}
				}
			}

			fclose(fp);
		}
	}
	*/
}

void tHumanTeamBaseState::mBeginState()
{
	FILE *fp = NULL;
#ifdef DEBUGMODE
	fp = fopen("C:\\Coop.txt", "a");
#endif

	if (fChildActor->mGetTeam() >= kTeam7)
		return;
	if ( fChildActor->mGetTeam() == kTeam1 ) {
		debug_fprintf(fp, "Mission has begun! Waiting for host's redalert.\n");
		mSetTimedInterval(kTimerSecondsSet, eBeginWait, 8.0f);
	}

	// All non-hosts should also register the enemy team state
	if (!gScriptInterface.mIsHost() && ( fChildActor->mGetTeam() == kTeam1 )) {
		debug_fprintf(fp, "Registering the enemy team's state.\n");
		for (int team = (int)kTeam7; team <= (int)kTeam10; team++)
		{
			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
			if (ti)
			{
				debug_fprintf(fp, "Team %d registered with non-host.\n", team);
				ti->mRegisterState( new tEnemyTeamBaseState() );
				ti->mGotoState( typeid( tEnemyTeamBaseState ) );
			}
		}
	}

#ifdef DEBUGMODE
	fclose(fp);
#endif
}


// STARTUP GETITEMS
void tHumanTeamBaseState::StartupGetItem(	tShipInfo* ship, eShipUpdateType type,
											eAccessShipStores_ItemTypes item, eAccessShipStores_SubTypes subItem, int32 value )
{
		eTeamID current_team = fChildActor->mGetTeam();
		// AI teams, don't come here.
		if (current_team >= kTeam7)
			return;

		// Bug? Each ship is passed through here FOR each player in game.
		// This should prevent extra passes for players and AI.
		if (ship->mGetTeam() != current_team)
			return;	

		FILE *fp = NULL;
#ifdef DEBUGMODE
		fp = fopen("C:\\Coop.txt", "a");

		debug_fprintf(fp, "LOADOUTS STEP 2: tHumanTeamBaseState::StartupGetItem\n");
#endif
		int32 bpv_to_add = 0;
		Ship aShip;
		if (
			(type == kShipCurrentValue) 
			&& ((item == kMissilesTypeI) || (item == kMissilesTypeIV))
			)
		{
			switch(item)
			{
			case kMissilesTypeI: 
				debug_fprintf(fp, "current value: item type is drone type 1\n");
				break;
			case kMissilesTypeIV: 
				debug_fprintf(fp, "current value: item type is drone type 4\n");
				break;
			default:
				debug_fprintf(fp, "current value: unknown item type.\n");
				break;
			}

			eAccessShipStores_SubTypes Drone_Speed;

			// latestShip was defined earlier. ;>
			if (latestShip > 7)
			{			// late era
				debug_fprintf(fp, "Setting player drones to fast, number: %d\n", value);
				Drone_Speed = kSpeed_Fast;
			}
			else if (latestShip >= 0)
			{			// mid era
				debug_fprintf(fp, "Setting player drones to med speed, number: %d\n", value);
				Drone_Speed = kSpeed_Med;
			} 
			else
			{			// early
				debug_fprintf(fp, "Setting player drones to med speed, number: %d\n", value);
				Drone_Speed = kStandard;
			}
			debug_fprintf(fp, "SETTING THE DRONE LOADOUT DOES NOT WORK.\n");
			debug_fprintf(fp, "I'm going to leave this in because I put so much work into it.\n");

			//ship->mRemoveItemFromStores( item,  value); // DOES NOT WORK
			//ship->mSetItemInStores( item, Drone_Speed, value); // DOES NOT WORK.
			//ship->mAddItemToStores( item, Drone_Speed, value); // DOES NOT WORK
		}
		if ((type == kShipCurrentValue) && (item == kMarines))
		{
			debug_fprintf(fp, "ship (%s) has %d marines.\n", ship->mGetClassName(), value);

			// Get the base value of same shipclass.
			// if #marines > base#marines
			//  then extraBPV = (#marines-base#marines) * 2
			selected_human_sl.Find(ship->mGetClassName(), aShip);
			{
				debug_fprintf(fp, "Base config of %s has %d marines.\n", 
					ship->mGetClassName(), aShip.marinesBase);

				if (value > aShip.marinesBase) {
					bpv_to_add = (value - aShip.marinesBase) * 2;
					debug_fprintf(fp, "BPV to add for enemy extras: %d.\n", 
						bpv_to_add);

					extras_bpv_to_assign += bpv_to_add;
					debug_fprintf(fp, "new total of extra enemy BPV: %d\n", 
							extras_bpv_to_assign);
				} else {
					debug_fprintf(fp, "No change to extra enemy BPV. Current total: %d\n",
						extras_bpv_to_assign);
				}
			}
		}
		else if ((type == kShipCurrentValue) && (item == kSmall_Mines))
		{
			debug_fprintf(fp, "ship (%s) has %d mines.\n", 
				ship->mGetClassName(), value);

			// Get the base value of same shipclass.
			// if #SmallMines> base#SmallMines
			//  then extraBPV = (#SmallMines-base#SmallMines) * 4
			selected_human_sl.Find(ship->mGetClassName(), aShip);
			{
				debug_fprintf(fp, "Base config of (%s) has %d tbombs.\n", 
					ship->mGetClassName(), aShip.tBombsBase);
				if (value > aShip.tBombsBase) {
					bpv_to_add = (value - aShip.tBombsBase) * 4;

					debug_fprintf(fp, "BPV to add for enemy extras: %d.\n", 
						bpv_to_add);
					extras_bpv_to_assign += bpv_to_add;

					debug_fprintf(fp, "new total of extra enemy BPV: %d\n", 
						extras_bpv_to_assign);
				} else {
					debug_fprintf(fp, "No change to extra enemy BPV. Current total: %d\n",
						extras_bpv_to_assign);
				}
			}
		}
		else if ((type == kShipCurrentValue) && (item == kSpareParts))
		{
			debug_fprintf(fp, "ship (%s) has %d spare parts.\n", 
				ship->mGetClassName(), value);
			selected_human_sl.Find(ship->mGetClassName(), aShip);

			debug_fprintf(fp, "Base config of (%s) has %d damageControl parts.\n", 
				ship->mGetClassName(), DamCon_to_SpareParts(aShip.damageControl));

			if (value > DamCon_to_SpareParts(aShip.damageControl)) 
			{
				bpv_to_add = 
					(value - DamCon_to_SpareParts(aShip.damageControl)) * 4;
				debug_fprintf(fp, "BPV to add for enemy extras: %d.\n", 
					bpv_to_add);

				extras_bpv_to_assign += bpv_to_add;
				debug_fprintf(fp, "new total of extra enemy BPV: %d\n", 
					extras_bpv_to_assign);
			} else {
				debug_fprintf(fp, "No change to extra enemy BPV. Current total: %d\n",
					extras_bpv_to_assign);
			}
		}
#ifdef DEBUGMODE
		fflush(fp);
		fclose(fp);
#endif
}


