/********************************************************************************************************\

	File Name:	MissionMaps.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionMaps.h"
#include "Mul_Coopace.h"

/*
	G-L are fixed starting positions for the 6 teams
*/
#ifdef OP
	int32 TotalMaps = 8;
#else
	int32 TotalMaps = 7;
#endif
const char* MapTypes[] = {	"Open Space",
							"Asteroid Field",
							"Nebula",
							"Black Hole",
							"Planet",
#ifdef OP
							"Ion Storms",
#endif
							"Dust Clouds",
							"Asteroid Belt",
							//"Moving Asteroids",
							};

/*
	Legend:
			Starting Positions			Ending Positions			Planets			
			Start1 --> G				End1 --> g					Earth --> 1
			Start2 --> H				End2 --> h					Ringed Earth --> 2
			Start3 --> I				End3 --> i					Mars --> 3
			Start4 --> J				End4 --> j					Jupiter --> 4
			Start5 --> K				End5 --> k					Organia --> 5
			Start6 --> L				End6 --> l					Saturn --> 6
			Start7 --> M				End7 --> m					Night City --> 7
			Start8 --> N				End8 --> n					Fire --> 8
			Start9 --> O				End9 --> o					Ice --> 9
			Start10 --> P				End10 --> p					Gas --> 0
			Start11 --> Q				End11 --> q					
			Start12 --> R				End12 --> r					
			Start13 --> S				End13 --> s					
			Start14 --> T				End14 --> t					
			Start15 --> U				End15 --> u					
			Start16 --> V				End16 --> v					
			Start17 --> W				End17 --> w					
			Start18 --> X				End18 --> x					
			Start19 --> Y				End19 --> y					
			Start20 --> Z				End20 --> z					

			Celestial Bodies			Asteroids					Black Holes
			Moon --> @					SmallAsteroid --> [			SmallBlackHole --> :
			Sun1 --> (					LargeAsteroid --> ]			LargeBlackHole --> ,
			Sun2 --> )					IceAsteroid --> *			
			Sun3 --> !
			
			Nebulas						Dust Clouds					Other
			Nebula1 --> &				LightDustCloud --> <		WormHole --> %
			Nebula2 --> ?				HeavyDustCloud --> >		Fissure --> $
																	Corpse --> #
																	Organian --> ^
																	Pulsar --> ~
																	IonStorm --> {
*/
const char* MapLines[]=
{
   "Mul_CoopAce",
   "// Open Space",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |.........P.N.M.O.........|",
   " |.........p.n.m.o.........|",
   " |.........................|",
   "2|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........ljhgik..........|",
   " |.........LJHGIK..........|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Field",
   " +____1____2____3____4____5+",
   " |............*............|",
   " |......*.........*........|",
   " |....................*....|",
   " |..*......................|",
   "1|.....**........*.........|",
   " |.....**..................|",
   " |.......P.*.N.M..O**....*.|",
   " |.......p...n.m..o**......|",
   " |...*.....................|",
   "2|.......*....*............|",
   " |....................*....|",
   " |................*........|",
   " |............*............|",
   " |.......*.................|",
   "3|.........................|",
   " |.........................|",
   " |....*....ljhgik....*...**|",
   " |.........LJHGIK........**|",
   " |.**...**.................|",
   "4|.**......................|",
   " |..........*...*..........|",
   " |...................**....|",
   " |.......*...........**....|",
   " |...*......*...*..........|",
   "5|........................*|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Nebula",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.&.......................|",
   " |.........................|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |.........P.N.M.O.........|",
   " |.........p.n.m.o.........|",
   " |.........................|",
   "2|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........ljhgik..........|",
   " |.........LJHGIK..........|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Black Hole",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |........P.N.M.O..........|",
   " |........p.n.m.o..........|",
   " |.........................|",
   "2|.........................|",
   " |.........................|",
   " |............,............|",	// comma = large black hole
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........ljhgik..........|",
   " |.........LJHGIK..........|",
   " |.........................|",
   "4|.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Planet",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.........................|",
   " |......................)..|",
   " |.........................|",
   "1|.........................|",
   " |.........................|",
   " |.........P.N.M.O.........|",
   " |.........p.n.m.o.........|",
   " |.........................|",
   "2|.........................|",
   " |.........................|",
   " |............7............|",	// night city
   " |.........................|",
   " |.........................|",
   "3|.........................|",
   " |.........................|",
   " |.........ljhgik..........|",
   " |.........LJHGIK..........|",
   " |.........................|",
   "4|.........................|",
   " |.[.......................|",
   " |..][.....................|",
   " |....][...................|",
   " |......][[................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
#ifdef OP
   "// Ion Storms",
   " +____1____2____3____4____5+",
   " |.........................|",
   " |.......{.................|",
   " |.........................|",
   " |..................{......|",
   "1|.........................|",
   " |.........................|",
   " |....{....P.N.M.O.........|",
   " |.........p.n.m.o.........|",
   " |.........................|",
   "2|..................{......|",
   " |.........................|",
   " |...........{.............|",
   " |..{......................|",
   " |.........................|",
   "3|.........................|",
   " |.....................{...|",
   " |.........ljhgik..........|",
   " |.........LJHGIK..........|",
   " |.........................|",
   "4|.........................|",
   " |.....{...................|",
   " |.........................|",
   " |..................{......|",
   " |.........................|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
#endif
   "// Dust Clouds",
   " +____1____2____3____4____5+",
   " |......<<<................|",
   " |......<><................|",
   " |......<<<........<<<.....|",
   " |.................<><.....|",
   "1|.................<<<.....|",
   " |...<<<...................|",
   " |...<><...P.N.M.O.........|",
   " |...<<<...p.n.m.o.........|",
   " |.................<<<.....|",
   "2|.................<><.....|",
   " |..........<<<....<<<.....|",
   " |..<<<.....<><............|",
   " |..<><.....<<<............|",
   " |..<<<....................|",
   "3|....................<<<..|",
   " |....................<><..|",
   " |.........ljhgik.....<<<..|",
   " |.........LJHGIK..........|",
   " |.........................|",
   "4|....<<<..................|",
   " |....<><..................|",
   " |....<<<..........<<<.....|",
   " |.................<><.....|",
   " |.................<<<.....|",
   "5|.........................|",
   " +-------------------------+",
   "      1    2    3    4    5",
   "// Asteroid Belt",
   " +____1____2____3____4____5+",
   " |...<*<...................|",
   " |.....<<<.................|",
   " |......<*<................|",
   " |[.......<<<..............|",
   "1|.........<*<.............|",
   " |...[...O...<<<...........|",
   " |........o...<*<..........|",
   " |..]...[.......<<<........|",
   " |[..........M...<*<.......|",
   "2|.....]...[..m....<<<.....|",
   " |...[..............<*<....|",
   " |........]...[...N...<<<..|",
   " |......[..........n...<*<.|",
   " |...........]...[.......<<|",
   "3|<<.......[..........P....|",
   " |<*<...........]...[..p...|",
   " |..<<<.......[............|",
   " |...<*<...........]...[...|",
   " |.....<<<.......[.........|",
   "4|......<*<...........]...[|",
   " |.lj.....<<<.......[......|",
   " |.LJhg....<*<...........].|",
   " |...HGik....<<<.......[...|",
   " |.....IK.....<*<..........|",
   "5|..............<<<.......[|",
   " +-------------------------+",
   "      1    2    3    4    5",
#if 0
   #// Moving Asteroids",
   " +____1____2____3+",
   " |...............|",
   " |...............|",
   " |....P.N.M.O....|",
   " |..[.p.n.m.o..[.|",
   "1|...............|",
   " |...............|",
   " |....[.....[....|",
   " |[.,....,....,.[|",
   " |...............|",
   "2|....[.....[....|",
   " |...............|",
   " |....ljhgik.....|",
   " |..[.LJHGIK...[.|",
   " |...............|",
   "3|...............|",
   " +---------------+",
   "      1    2     5",
#endif
   "///",
   0
};
